//Om Ganesayanamaha
package org.processmining.plugins.abstractions;

import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JPanel;

import org.jgraph.graph.DefaultEdge;
import org.jgrapht.alg.DijkstraShortestPath;
import org.jgrapht.graph.SimpleDirectedGraph;
import org.processmining.plugins.abstractions.datatypes.EncodedLog;
import org.processmining.plugins.abstractions.enumtypes.AbstractionStrategy;
import org.processmining.plugins.abstractions.enumtypes.RemoveAbstractionStrategy;
import org.processmining.plugins.abstractions.listeners.AbstractionElementListener;
import org.processmining.plugins.abstractions.util.AbstractionSetTheory;
import org.processmining.plugins.abstractions.visualization.AbstractionElementComponent;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be
 *       done with proper acknowledgments by citing "R. P. Jagadeesh Chandra
 *       Bose and Wil M. P. van der Aalst. Abstractions in Process Mining: A
 *       Taxonomy of Patterns. Proceedings of the International Conference on
 *       Business Process Management (BPM). Volume 5701 of LNCS.,
 *       Springer-Verlag (2009) pages 159-175"
 * 
 */
public class MineAbstractions implements AbstractionElementListener {
	AbstractionStrategy abstractionStrategy;
	AbstractionSetTheory ast;
	JPanel abstractionResultsPanel;
	EncodedLog encodedLog;
	
	boolean isExtendMaximalElements;
	float commonElementsThreshold;
	int minMaximalElementSizeThreshold;
	
	List<TreeSet<String>> patternAlphabetList;
	Set<Set<String>> patternAlphabetSet;
	Map<Set<String>, Set<Set<String>>> originalMaximalElementSubsumedPatternAlphabetSet;
	Map<Set<String>, Set<Set<String>>> maximalElementSubsumedPatternAlphabetSetMap;
	
	Map<Set<String>, Set<Set<String>>> mergedMaximalElementSubsumedMaximalElementSetMap;
	Set<Set<String>> removedPatternAlphabetSet;
	Map<Set<String>, Set<String>> originalPatternAlphabetModifiedPatternAlphabetMap;
	
	public MineAbstractions(EncodedLog encodedLog, JPanel abstractionResultsPanel){
		this.encodedLog = encodedLog;
		this.abstractionResultsPanel = abstractionResultsPanel;
	
		originalPatternAlphabetModifiedPatternAlphabetMap = new HashMap<Set<String>, Set<String>>();
		mergedMaximalElementSubsumedMaximalElementSetMap = new HashMap<Set<String>, Set<Set<String>>>();
		removedPatternAlphabetSet = new HashSet<Set<String>>();
	}
	
	public void reset(){
		originalMaximalElementSubsumedPatternAlphabetSet.clear();
		mergedMaximalElementSubsumedMaximalElementSetMap.clear();
		removedPatternAlphabetSet.clear();
	}
	
	public void setAbstractionStrategy(AbstractionStrategy abstractionStrategy){
		this.abstractionStrategy = abstractionStrategy;
	}
	
	public void mineSetTheoryAbstractions(List<TreeSet<String>> patternAlphabetList, boolean isExtendMaximalElements, float commonElementsThreshold, int minMaximalElementSizeThreshold){
		mergedMaximalElementSubsumedMaximalElementSetMap.clear();
		removedPatternAlphabetSet.clear();
		originalPatternAlphabetModifiedPatternAlphabetMap.clear();
		
		this.isExtendMaximalElements = isExtendMaximalElements;
		this.commonElementsThreshold = commonElementsThreshold;
		this.minMaximalElementSizeThreshold = minMaximalElementSizeThreshold;
		
		patternAlphabetSet = new HashSet<Set<String>>();
		patternAlphabetSet.addAll(patternAlphabetList);
		
		ast = new AbstractionSetTheory(patternAlphabetSet);
		if(isExtendMaximalElements){
			ast.extendMaximalElements(minMaximalElementSizeThreshold, commonElementsThreshold);
			ast.printGraph();
		}
		ast.preparePatternAlphabetMaximalElementRelations();
		originalMaximalElementSubsumedPatternAlphabetSet = ast.getMaximalElementSubsumedAlphabetSetMap();
		maximalElementSubsumedPatternAlphabetSetMap = ast.getMaximalElementSubsumedAlphabetSetMap();
		buildUI();
	}
	
	public void mineSetTheoryAbstractions(Set<Set<String>> patternAlphabetSet){
		ast = new AbstractionSetTheory(patternAlphabetSet);
		if(isExtendMaximalElements){
			ast.extendMaximalElements(minMaximalElementSizeThreshold, commonElementsThreshold);
//			ast.printGraph();
		}
		ast.preparePatternAlphabetMaximalElementRelations();
		maximalElementSubsumedPatternAlphabetSetMap = ast.getMaximalElementSubsumedAlphabetSetMap();
		buildUI();
	}
	
	private void buildUI(){
		abstractionResultsPanel.removeAll();
		switch(abstractionStrategy){
		case SetTheory:
			List<Set<String>> maximalElementList = ast.getMaximalElements();
			int noMaximalElements = maximalElementList.size();
			for(int i = 0; i < noMaximalElements; i++){
				AbstractionElementComponent aec = new AbstractionElementComponent("Abs "+i, maximalElementList.get(i), encodedLog);
				aec.setSelected(false);
				aec.addListener(this);
				abstractionResultsPanel.add(aec);
			}
			abstractionResultsPanel.revalidate();
			abstractionResultsPanel.repaint();
			break;
		case GraphTheory:
			break;
		}
	}

	//Abstraction Element Listener
	public void abstractionActivityModified(Set<String> originalAlphabet,
			Set<String> modifiedAlphabet, boolean applyGlobally) {
		if(originalAlphabet.equals(modifiedAlphabet))
			return;
		Set<Set<String>> subsumedAlphabetSet = new HashSet<Set<String>>();
		Set<Set<String>> origAlphabetSetSet = new HashSet<Set<String>>();
		origAlphabetSetSet.add(originalAlphabet);
		subsumedAlphabetSet = ast.getSubsumedPatternAlphabetSet(origAlphabetSetSet);
		Set<String> modifiedPatternAlphabet;
		
		Set<String> removedAlphabetSet = new HashSet<String>();
		removedAlphabetSet.addAll(originalAlphabet);
		removedAlphabetSet.removeAll(modifiedAlphabet);
		
		Set<String> tempSet = new HashSet<String>();
		Set<Set<String>> toRemovePatternAlphabetSet = new HashSet<Set<String>>();
		Set<Set<String>> toAddPatternAlphabetSet = new HashSet<Set<String>>();
		if(applyGlobally){
			for(Set<String> patternAlphabet : patternAlphabetSet){
				tempSet.clear();
				tempSet.addAll(patternAlphabet);
				tempSet.removeAll(removedAlphabetSet);
				tempSet.retainAll(originalAlphabet);
				if(tempSet.size() > 0){
					modifiedPatternAlphabet = new HashSet<String>();
					modifiedPatternAlphabet.addAll(patternAlphabet);
					if(modifiedPatternAlphabet.equals(removedAlphabetSet))
						continue;
					modifiedPatternAlphabet.removeAll(removedAlphabetSet);
					if(patternAlphabet.equals(modifiedPatternAlphabet))
						continue;
					originalPatternAlphabetModifiedPatternAlphabetMap.put(patternAlphabet, modifiedPatternAlphabet);
					toRemovePatternAlphabetSet.add(patternAlphabet);
					toAddPatternAlphabetSet.add(modifiedPatternAlphabet);
				}
			}
			patternAlphabetSet.removeAll(toRemovePatternAlphabetSet);
			patternAlphabetSet.addAll(toAddPatternAlphabetSet);
		}else{
			for(Set<String> patternAlphabet : subsumedAlphabetSet){
				modifiedPatternAlphabet = new HashSet<String>();
				modifiedPatternAlphabet.addAll(patternAlphabet);
				if(modifiedPatternAlphabet.equals(removedAlphabetSet))
					continue;
				modifiedPatternAlphabet.removeAll(removedAlphabetSet);
				if(patternAlphabet.equals(modifiedPatternAlphabet))
					continue;
				originalPatternAlphabetModifiedPatternAlphabetMap.put(patternAlphabet, modifiedPatternAlphabet);
				patternAlphabetSet.remove(patternAlphabet);
				if(modifiedPatternAlphabet.size() != 0)
					patternAlphabetSet.add(modifiedPatternAlphabet);
			}
		}
		originalPatternAlphabetModifiedPatternAlphabetMap.put(originalAlphabet, modifiedAlphabet);
		patternAlphabetSet.remove(originalAlphabet);
		patternAlphabetSet.add(modifiedAlphabet);
		
		
		mineSetTheoryAbstractions(patternAlphabetSet);
	}
	
	public void mergeMaximalElements(Set<Set<String>> maximalElementSet){
		Set<Set<String>> subsumedMaximalElementSet = new HashSet<Set<String>>();
		Set<String> mergedMaximalElement = new HashSet<String>();
		
		for(Set<String> maximalElement : maximalElementSet){
			mergedMaximalElement.addAll(maximalElement);
			if(mergedMaximalElementSubsumedMaximalElementSetMap.containsKey(maximalElement)){
				subsumedMaximalElementSet.addAll(mergedMaximalElementSubsumedMaximalElementSetMap.get(maximalElement));
				mergedMaximalElementSubsumedMaximalElementSetMap.remove(maximalElement);
			}else{
				subsumedMaximalElementSet.add(maximalElement);
			}
		}
		mergedMaximalElementSubsumedMaximalElementSetMap.put(mergedMaximalElement, subsumedMaximalElementSet);
		patternAlphabetSet.add(mergedMaximalElement);
		mineSetTheoryAbstractions(patternAlphabetSet);
	}
	
	public void removeMaximalElements(Set<Set<String>> maximalElementSet, RemoveAbstractionStrategy removeAbstractionStrategy){
		//Just rely on the AST and get all pattern alphabets uniquely subsumed in the maximal elements
		Set<Set<String>> toRemovePatternAlphabetSet;
		
		if(removeAbstractionStrategy == RemoveAbstractionStrategy.UNIQUELY_SUBSUMED_ALPHABETS){
			toRemovePatternAlphabetSet = ast.getUniquelySubsumedPatternAlphabetSet(maximalElementSet);
		}else{
			toRemovePatternAlphabetSet = new HashSet<Set<String>>();
		}
		
		toRemovePatternAlphabetSet.addAll(maximalElementSet);
		patternAlphabetSet.removeAll(toRemovePatternAlphabetSet);
		removedPatternAlphabetSet.addAll(toRemovePatternAlphabetSet);
		mineSetTheoryAbstractions(patternAlphabetSet);
	}

	public Map<Set<String>, TreeSet<String>> processPatternModifications(Map<TreeSet<String>, TreeSet<String>> patternAlphabetPatternSetMap){
		SimpleDirectedGraph<Set<String>, DefaultEdge> sg = new SimpleDirectedGraph<Set<String>, DefaultEdge>(DefaultEdge.class);
		try{
			for(Set<String> originalAlphabet : originalPatternAlphabetModifiedPatternAlphabetMap.keySet()){
				sg.addVertex(originalAlphabet);
				sg.addVertex(originalPatternAlphabetModifiedPatternAlphabetMap.get(originalAlphabet));
				sg.addEdge(originalAlphabet, originalPatternAlphabetModifiedPatternAlphabetMap.get(originalAlphabet));
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		
		Set<Set<String>> finalModifiedAlphabetSet = new HashSet<Set<String>>();
		for(Set<String> vertex : sg.vertexSet())
			if(sg.outDegreeOf(vertex) == 0)
				finalModifiedAlphabetSet.add(vertex);
		
		Set<Set<String>> finalOriginalAlphabetSet = new HashSet<Set<String>>();
		for(Set<String> vertex : sg.vertexSet())
			if(sg.inDegreeOf(vertex) == 0)
				finalOriginalAlphabetSet.add(vertex);
		
		Map<Set<String>, Set<String>> finalOriginalAlphabetModifiedAlphabetMap = new HashMap<Set<String>, Set<String>>();
		
		int noModifiedAlphabets;

		Set<String> modifiedAlphabet = null;
		for(Set<String> finalOriginalAlphabet : finalOriginalAlphabetSet){
			noModifiedAlphabets = 0;
			
			for(Set<String> finalModifiedAlphabet : finalModifiedAlphabetSet){
				DijkstraShortestPath<Set<String>, DefaultEdge> shortestPath = new DijkstraShortestPath<Set<String>, DefaultEdge>(
						sg, finalOriginalAlphabet, finalModifiedAlphabet);
				if(shortestPath.getPathEdgeList() != null){
					noModifiedAlphabets++;
					modifiedAlphabet = finalModifiedAlphabet;
				}
			}
			if(noModifiedAlphabets != 1){
				System.out.println("Alphabet "+finalOriginalAlphabet+" has more than one modified alphabet");
			}else{
				if(!(removedPatternAlphabetSet.contains(finalOriginalAlphabet) || removedPatternAlphabetSet.contains(modifiedAlphabet)))
					finalOriginalAlphabetModifiedAlphabetMap.put(finalOriginalAlphabet, modifiedAlphabet);
			}
		}
		
		//Print the final orinal alphabet, modified alphabet
	
		Map<Set<String>, Set<Set<String>>> modifiedAlphabetOriginalAlphabetSetMap = new HashMap<Set<String>, Set<Set<String>>>();
		Set<Set<String>> modifiedAlphabetOriginalAlphabetSet;
		for(Set<String> finalOriginalAlphabet : finalOriginalAlphabetModifiedAlphabetMap.keySet()){
			modifiedAlphabet = finalOriginalAlphabetModifiedAlphabetMap.get(finalOriginalAlphabet);
			if(modifiedAlphabetOriginalAlphabetSetMap.containsKey(modifiedAlphabet)){
				modifiedAlphabetOriginalAlphabetSet = modifiedAlphabetOriginalAlphabetSetMap.get(modifiedAlphabet);
			}else{
				modifiedAlphabetOriginalAlphabetSet = new HashSet<Set<String>>();
			}
			modifiedAlphabetOriginalAlphabetSet.add(finalOriginalAlphabet);
			modifiedAlphabetOriginalAlphabetSetMap.put(modifiedAlphabet, modifiedAlphabetOriginalAlphabetSet);
		}
		
		/*
		 * For each pattern alphabet, get the set of abstraction names; it could
		 * be the case that a pattern alphabet is subsumed in more than one
		 * maximal element, each of which can have a different name
		 */
		
		//Get the vertices of the ast
		Set<Set<String>> vertexSet = ast.getVertexSet();
		Map<Set<String>, TreeSet<String>> filteredPatternAlphabetPatternSetMap = new HashMap<Set<String>, TreeSet<String>>();
		TreeSet<String> filteredPatternAlphabetPatternSet;
		Set<String> originalAlphabetPatternSet;
		Set<String> modifiedAlphabetPatternSet;
		
		Set<String> removedActivitySet = new HashSet<String>();
		Set<Set<String>> toConsiderPatternAlphabetSet = new HashSet<Set<String>>();
		
		for(Set<String> vertex : vertexSet){
			//First check if the vertex is a result of merging other maximal elements; if so, process for each merged maximal element
			toConsiderPatternAlphabetSet.clear();
			if(mergedMaximalElementSubsumedMaximalElementSetMap.containsKey(vertex)){
				toConsiderPatternAlphabetSet.addAll(mergedMaximalElementSubsumedMaximalElementSetMap.get(vertex));
			}else{
				toConsiderPatternAlphabetSet.add(vertex);
			}
			for(Set<String> patternAlphabet : toConsiderPatternAlphabetSet){
				if(patternAlphabetPatternSetMap.containsKey(patternAlphabet)){
					if(filteredPatternAlphabetPatternSetMap.containsKey(patternAlphabet))
						filteredPatternAlphabetPatternSet = filteredPatternAlphabetPatternSetMap.get(patternAlphabet);
					else
						filteredPatternAlphabetPatternSet = new TreeSet<String>(new DescendingLengthComparator());
					
					filteredPatternAlphabetPatternSet.addAll((patternAlphabetPatternSetMap.get(patternAlphabet)));
					filteredPatternAlphabetPatternSetMap.put(patternAlphabet, filteredPatternAlphabetPatternSet);
				}else{
					if(modifiedAlphabetOriginalAlphabetSetMap.containsKey(patternAlphabet)){
						modifiedAlphabetPatternSet = new TreeSet<String>();
						modifiedAlphabetOriginalAlphabetSet = modifiedAlphabetOriginalAlphabetSetMap.get(patternAlphabet);
						for(Set<String> originalAlphabet : modifiedAlphabetOriginalAlphabetSet){
							if(patternAlphabetPatternSetMap.containsKey(originalAlphabet)){
								originalAlphabetPatternSet = patternAlphabetPatternSetMap.get(originalAlphabet);
								removedActivitySet.clear();
								removedActivitySet.addAll(originalAlphabet);
								removedActivitySet.removeAll(patternAlphabet);
								
								for(String pattern : originalAlphabetPatternSet){
									for(String removedActivity : removedActivitySet)
										pattern = pattern.replaceAll(removedActivity, "");
									
									modifiedAlphabetPatternSet.add(pattern);
								}
								
							}else{
								System.out.println("Shouldn't be the case; original alphabet not in patternAlphabetPatternSet");
							}
						}
						if(filteredPatternAlphabetPatternSetMap.containsKey(patternAlphabet))
							filteredPatternAlphabetPatternSet = filteredPatternAlphabetPatternSetMap.get(patternAlphabet);
						else
							filteredPatternAlphabetPatternSet = new TreeSet<String>(new DescendingLengthComparator());
						filteredPatternAlphabetPatternSet.addAll(modifiedAlphabetPatternSet);
						if(filteredPatternAlphabetPatternSet.size() > 0){
							filteredPatternAlphabetPatternSetMap.put(patternAlphabet, filteredPatternAlphabetPatternSet);
						}
					}else{
						System.out.println("Shouldn't be the case; vertex is neither in pattern alphabet set nor in modified alphabet set: "+vertex);
					}
				}
			}
		}
		
		return filteredPatternAlphabetPatternSetMap;
	}

	
	
	class DescendingLengthComparator implements Comparator<String>{
		public int compare(String str1, String str2) {
			int lengthDiff = str2.length()-str1.length();
			return (lengthDiff != 0? lengthDiff : str2.compareTo(str1));
		}
	}



	public Map<Set<String>, Set<Set<String>>> getMergedMaximalElementSubsumedMaximalElementSetMap() {
		return mergedMaximalElementSubsumedMaximalElementSetMap;
	}

	public Map<Set<String>, Set<Set<String>>> getMaximalElementSubsumedPatternAlphabetSetMap() {
		return maximalElementSubsumedPatternAlphabetSetMap;
	}
}
