package org.processmining.plugins.abstractions;

import java.util.Map;
import java.util.Set;

import org.processmining.plugins.abstractions.enumtypes.AbstractionStrategy;
import org.processmining.plugins.abstractions.enumtypes.PatternFilter;
import org.processmining.plugins.abstractions.enumtypes.PatternLengthPreference;
import org.processmining.plugins.abstractions.enumtypes.PatternMetric;
import org.processmining.plugins.abstractions.enumtypes.PatternType;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 * 
 */
public class PatternAbstractionsInput {
	Set<PatternType> patternTypeSet;
	Set<PatternMetric> patternMetricSet;
	Map<PatternFilter, Float> patternFilterThresholdMap;
	PatternLengthPreference patternLengthPreference;

	int splitTraceThreshold;
	
	boolean isBasePatterns;
	boolean isSplitTrace;
	boolean isPrunePatterns;
	boolean ignoreDuplicateTraces;
	
	AbstractionStrategy abstractionStrategy;
	boolean isMergeMaximalElements;
	float commonElementsPercentage;
	float diffElementsPercentage;
	
	public PatternAbstractionsInput(){
		
	}
	
	public void setPatternTypeSelection(Set<PatternType> patternTypeSet){
		this.patternTypeSet = patternTypeSet;
	}
	
	public void setPatternMetricSelection(Set<PatternMetric> patternMetricSet){
		this.patternMetricSet = patternMetricSet;
	}
	
	public void setPatternFilterSelection(Map<PatternFilter, Float> patternFilterThresholdMap){
		this.patternFilterThresholdMap = patternFilterThresholdMap;
	}
	
	public void setPatternLengthPreference(PatternLengthPreference patternLengthPreference){
		this.patternLengthPreference = patternLengthPreference;
	}
	
	public void setIsBasePatterns(boolean isBasePatterns){
		this.isBasePatterns = isBasePatterns;
	}
	
	public void setIsPrunePatterns(boolean isPrunePatterns){
		this.isPrunePatterns = isPrunePatterns;
	}
	
	public void setSplitTraceThreshold(int splitTraceThreshold){
		this.splitTraceThreshold = splitTraceThreshold;
	}

	public Set<PatternType> getPatternTypeSet() {
		return patternTypeSet;
	}

	public Set<PatternMetric> getPatternMetricSet() {
		return patternMetricSet;
	}

	public Map<PatternFilter, Float> getPatternFilterThresholdMap() {
		return patternFilterThresholdMap;
	}

	public PatternLengthPreference getPatternLengthPreference() {
		return patternLengthPreference;
	}

	public int getSplitTraceThreshold() {
		return splitTraceThreshold;
	}

	public boolean isBasePatterns(){
		return isBasePatterns;
	}
	
	public boolean isPrunePatterns() {
		return isPrunePatterns;
	}

	public void setAbstractionStrategy(AbstractionStrategy abstractionStrategy) {
		this.abstractionStrategy = abstractionStrategy;
	}

	public void setCommonElementsPercentage(float commonElementsPercentage) {
		this.commonElementsPercentage = commonElementsPercentage;
	}

	public void setDiffElementsPercentage(float diffElementsPercentage) {
		this.diffElementsPercentage = diffElementsPercentage;
	}
	
	public AbstractionStrategy getAbstractionStrategy() {
		return abstractionStrategy;
	}

	public float getCommonElementsPercentage() {
		return commonElementsPercentage;
	}

	public float getDiffElementsPercentage() {
		return diffElementsPercentage;
	}
}

