package org.processmining.plugins.abstractions.datatypes;

import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.processmining.plugins.abstractions.util.FileIO;
import org.processmining.plugins.abstractions.util.Logger;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 * 
 */
public class EncodedLog {
	int encodingLength;
	long totalTraceLength;
	long averageTraceLength;
	int maximumTraceLength;
	int maximumActivityNameLength;
	
	Map<String, String> activityCharMap;
	Map<String, String> charActivityMap;
	List<String> encodedTraceList;
	List<String> uniqueEncodedTraceList;
	Map<String, TreeSet<Integer>> encodedTraceIdenticalTraceIndicesSetMap;
	
	public EncodedLog(){
		
	}

	public void setEncodingLength(int encodingLength) {
		this.encodingLength = encodingLength;
	}

	public void setAverageTraceLength(long averageTraceLength) {
		this.averageTraceLength = averageTraceLength;
	}

	public void setMaximumActivityNameLength(int maximumActivityNameLength) {
		this.maximumActivityNameLength = maximumActivityNameLength;
	}

	public void setActivityCharMap(Map<String, String> activityCharMap) {
		this.activityCharMap = activityCharMap;
	}

	public void setCharActivityMap(Map<String, String> charActivityMap) {
		this.charActivityMap = charActivityMap;
	}

	public void setEncodedTraceList(List<String> encodedTraceList) {
		this.encodedTraceList = encodedTraceList;
	}

	public void setUniqueEncodedTraceList(List<String> uniqueEncodedTraceList) {
		this.uniqueEncodedTraceList = uniqueEncodedTraceList;
	}

	public void setTotalTraceLength(long totalTraceLength) {
		this.totalTraceLength = totalTraceLength;
	}
	
	public void setMaximumTraceLength(int maximumTraceLength){
		this.maximumTraceLength = maximumTraceLength;
	}
	
	public int getMaximumTraceLength(){
		return maximumTraceLength;
	}
	
	public int getNoTraces(){
		return encodedTraceList.size();
	}
	
	public int getNoUniqueTraces(){
		return uniqueEncodedTraceList.size();
	}
	
	public long getTotalTraceLength() {
		return this.totalTraceLength;
	}
	
	public void setEncodedTraceIdenticalTraceIndicesSetMap(
			Map<String, TreeSet<Integer>> encodedTraceIdenticalTraceIndicesSetMap) {
		this.encodedTraceIdenticalTraceIndicesSetMap = encodedTraceIdenticalTraceIndicesSetMap;
	}
	
	public int getEncodingLength() {
		return encodingLength;
	}

	public long getAverageTraceLength() {
		return averageTraceLength;
	}

	public int getMaximumActivityNameLength() {
		return maximumActivityNameLength;
	}

	public Map<String, String> getActivityCharMap() {
		return activityCharMap;
	}

	public Map<String, String> getCharActivityMap() {
		return charActivityMap;
	}

	public List<String> getEncodedTraceList() {
		return encodedTraceList;
	}

	public List<String> getUniqueEncodedTraceList() {
		return uniqueEncodedTraceList;
	}

	public Map<String, TreeSet<Integer>> getEncodedTraceIdenticalTraceIndicesSetMap() {
		return encodedTraceIdenticalTraceIndicesSetMap;
	}
	
	public void print(){
		System.out.println("Encoding Length: "+encodingLength);
		System.out.println("No. Traces: "+encodedTraceList.size());
		System.out.println("No. Unique Traces: "+uniqueEncodedTraceList.size());
		System.out.println("Avg. Trace Length: "+averageTraceLength);
		System.out.println("Total Trace Length: "+totalTraceLength);
	}
	
	public void printLog(){
		String fileSeparator = System.getProperty("file.separator");
		String tempDir = System.getProperty("java.io.tmpdir")+fileSeparator+"Abstractions"+fileSeparator+"MinePatterns";
		String delim = "@";
		
		FileIO io = new FileIO();
		io.writeToFile(tempDir, "EncodedTraceList.txt", encodedTraceList);
		io.writeToFile(tempDir, "UniqueEncodedTraceList.txt", uniqueEncodedTraceList);
		io.writeToFile(tempDir, "CharActivityMap.txt", charActivityMap, delim);
		io.writeToFile(tempDir, "ActivityCharMap.txt", activityCharMap, delim);
		
		Logger.println("Encoding Length: "+encodingLength);
		Logger.println("No. Traces: "+encodedTraceList.size());
		Logger.println("No. Unique Traces: "+uniqueEncodedTraceList.size());
		Logger.println("Avg. Trace Length: "+averageTraceLength);
	}
}
