package org.processmining.plugins.abstractions.encoding;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.abstractions.exceptions.EncodingNotFoundException;
import org.processmining.plugins.abstractions.util.LogUtils;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 * 
 */
public class EncodeTraces {
	private final ArrayList<String> charStreamList;
	private final Map<String, Integer> encodedTraceCountMap;

	public EncodeTraces(Map<String, String> activityCharMap, XLog log) throws EncodingNotFoundException {
		charStreamList = new ArrayList<String>();
		encodedTraceCountMap = new HashMap<String, Integer>();

		StringBuilder charStreamBuilder = new StringBuilder();
		StringBuilder activityBuilder = new StringBuilder();

		int count;
		for (XTrace trace : log) {
			charStreamBuilder.setLength(0);
			for (XEvent event : trace) {
				activityBuilder.setLength(0);
				
				activityBuilder.append(LogUtils.getActivity(event));

				if (activityCharMap.containsKey(activityBuilder.toString())) {
					charStreamBuilder.append(activityCharMap.get(activityBuilder.toString()));
				} else {
					throw new EncodingNotFoundException(activityBuilder.toString());
				}
			}
			count = 1;
			charStreamList.add(charStreamBuilder.toString());
			if(encodedTraceCountMap.containsKey(charStreamBuilder.toString())){
				count += encodedTraceCountMap.get(charStreamBuilder.toString());
			}
			encodedTraceCountMap.put(charStreamBuilder.toString(), count);
		}
	}

	public ArrayList<String> getCharStreamList() {
		return charStreamList;
	}
	
	public List<String> getEncodedTraceList(){
		return charStreamList;
	}
	
	public Map<String, Integer> getEncodedTraceCountMap(){
		return encodedTraceCountMap;
	}
}
