package org.processmining.plugins.abstractions.visualization;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.processmining.plugins.abstractions.datatypes.EncodedLog;
import org.processmining.plugins.abstractions.listeners.AbstractionElementListener;
import org.processmining.plugins.abstractions.util.UkkonenSuffixTree;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 27 May 2010 (last modified)
 * @since 01 July 2009
 * @version 1.0 
 * @(c)R. P. Jagadeesh Chandra 'JC' Bose, Deparment of Mathematics
 *          and Computer Science, TU/e
 * 
 *  	 Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 */
@SuppressWarnings("serial")
public class AbstractionElementComponent extends JPanel {
	/**
	 * This comparator implements the functionality to facilitate the sorting of
	 * strings in descending order of their lengths
	 */
	class DescendingLengthComparator implements Comparator<String>{
		public int compare(String str1, String str2) {
			int lengthDiff = str2.length()-str1.length();
			return (lengthDiff != 0? lengthDiff : str2.compareTo(str1));
		}
	}
	
	public JCheckBox activeBox;
	protected JTextArea abstractionNameTextArea;
	protected JList abstractionActivitiesList;
	protected Set<String> originalAbstractionActivitySet;

	public Set<String> currentEncodedActivitySet;
	Set<String> currentDecodedActivitySet;
	Set<String> removedActivitySet;
	
	Map<String, String> activityCharMap;
	Map<String, String> charActivityMap;
	
	String abstractionName;	
	boolean hasModified = false;
	List<AbstractionElementListener> listenersList = new ArrayList<AbstractionElementListener>();

	Color backgroundColor = new Color(160,160,160);
	
	public AbstractionElementComponent(String abstractionName, Set<String> activitySet, EncodedLog encodedLog){
		super();
		setBorder(BorderFactory.createEtchedBorder());
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		this.abstractionName = abstractionName;
		this.activityCharMap = encodedLog.getActivityCharMap();
		this.charActivityMap = encodedLog.getCharActivityMap();
		
		activeBox = SlickerFactory.instance().createCheckBox("", false);
		this.add(activeBox);
		this.add(Box.createHorizontalStrut(15));
		
		this.setBackground(backgroundColor);
		
		originalAbstractionActivitySet = new HashSet<String>();
		originalAbstractionActivitySet.addAll(activitySet);
		
		currentEncodedActivitySet = new HashSet<String>();
		currentDecodedActivitySet = new HashSet<String>();
		
		for(String activity : activitySet){
			currentDecodedActivitySet.add(charActivityMap.get(activity));
			currentEncodedActivitySet.add(activity);
		}
		
		abstractionNameTextArea = new JTextArea(determineAbstractionName());
		abstractionNameTextArea.setBackground(backgroundColor);
		abstractionNameTextArea.setForeground(Color.blue);
		abstractionNameTextArea.setLineWrap(true);
		abstractionNameTextArea.setEditable(true);
		
		FontMetrics fm = getFontMetrics(getFont());
		abstractionNameTextArea.setPreferredSize(new Dimension(fm.stringWidth("A")*10, currentDecodedActivitySet.size()*(fm.getMaxAscent()+fm.getMaxDescent())));
		
		abstractionActivitiesList = new JList(currentDecodedActivitySet.toArray());
		abstractionActivitiesList.setAutoscrolls(true);
		abstractionActivitiesList.setBackground(backgroundColor);
		abstractionActivitiesList.setBorder(BorderFactory.createEtchedBorder());
		abstractionActivitiesList.setAlignmentX(LEFT_ALIGNMENT);
		abstractionActivitiesList.setPreferredSize(new Dimension(fm.stringWidth("A")*encodedLog.getMaximumActivityNameLength(), currentDecodedActivitySet.size()*(fm.getMaxAscent()+fm.getMaxDescent()+5)));
		
		MouseListener[] mouseListeners = abstractionActivitiesList.getMouseListeners();
		for(MouseListener mouseListerner : mouseListeners)
			abstractionActivitiesList.removeMouseListener(mouseListerner);
		
		abstractionActivitiesList.addMouseListener(new MouseAdapter() {
			 public void mouseClicked(MouseEvent e) {
				 	int index  = abstractionActivitiesList.locationToIndex(e.getPoint());
				 	if(abstractionActivitiesList.isSelectedIndex(index)){
				 		abstractionActivitiesList.removeSelectionInterval(index, index);
				 	}else{
				 		abstractionActivitiesList.addSelectionInterval(index, index);
				 	}
			    }
		});
		
		removedActivitySet = new HashSet<String>();
		JButton removeButton = SlickerFactory.instance().createButton("Remove");
		JButton splitButton = SlickerFactory.instance().createButton("Split");
		
		this.add(abstractionNameTextArea);
		this.add(Box.createHorizontalStrut(5));
		this.add(abstractionActivitiesList);
		this.add(Box.createHorizontalStrut(15));
		this.add(removeButton);
		this.add(Box.createHorizontalStrut(15));
		this.add(splitButton);
		this.add(Box.createGlue());
	
		removeButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				int confirmDialogResponse = confirmGlobalApplication();
				boolean applyGlobally;
				if(confirmDialogResponse == JOptionPane.YES_OPTION)
					applyGlobally = true;
				else if(confirmDialogResponse == JOptionPane.NO_OPTION)
					applyGlobally = false;
				else
					return;
				
				Object[] selectedActivitiesArray = abstractionActivitiesList.getSelectedValues(); 
				for(Object selectedActivity : selectedActivitiesArray)
					removedActivitySet.add((String)selectedActivity);
				
				System.out.print("Original Decoded Alphabet: "+currentDecodedActivitySet+" @ Modified: ");
				currentDecodedActivitySet.removeAll(removedActivitySet);
				System.out.println(currentDecodedActivitySet);
				currentEncodedActivitySet.clear();
				for(String decodedActivity : currentDecodedActivitySet)
					currentEncodedActivitySet.add(activityCharMap.get(decodedActivity));
				abstractionActivitiesList.setListData(currentDecodedActivitySet.toArray());
				hasModified = true;
				for(AbstractionElementListener listener : listenersList)
					listener.abstractionActivityModified(originalAbstractionActivitySet, currentEncodedActivitySet, applyGlobally);
			}
		});
		
		splitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		});
	}
	
	/**
	 * This method determines an apt abstraction name based on the activities defining this activity set
	 * @return a string most apt to be an abstraction name
	 */
	protected String determineAbstractionName(){
		if(currentDecodedActivitySet.size() == 1){
			 abstractionName = currentDecodedActivitySet.toArray()[0].toString();
			 abstractionName = abstractionName.replaceAll("-start", "").replaceAll("-complete", "");
		}else{
			String[] delimiterArray = {"~", "!", "@", "#", "%", "^", "&", "*"};
			int index = 0;
			StringBuilder strBuilder = new StringBuilder();
			for(String decodedActivity : currentDecodedActivitySet){
				strBuilder.append(decodedActivity).append(delimiterArray[index%delimiterArray.length]);
				index++;
			}
			UkkonenSuffixTree suffixTree = new UkkonenSuffixTree(1, strBuilder.toString());
			suffixTree.findLeftDiverseNodes();
			Set<String> maximalRepeatSet = suffixTree.getSuperMaximalRepeats();
			TreeSet<String> filteredMaximalRepeatSet = new TreeSet<String>(new DescendingLengthComparator());
			if (maximalRepeatSet != null) {
				for (String repeat : maximalRepeatSet)
					if (!(repeat.contains("complete") || repeat.contains("start")) && repeat.length() >= 4)
						filteredMaximalRepeatSet.add(repeat.replaceAll("-", ""));
			}
			if(filteredMaximalRepeatSet.size() > 0)
				abstractionName = filteredMaximalRepeatSet.toString();
		}
		return abstractionName;
	}
	
	private int confirmGlobalApplication(){
		return JOptionPane.showConfirmDialog(this, "Do you want to filter this combination of activities across all patterns?", "Confirm", JOptionPane.YES_NO_OPTION);
	}
	
	public boolean hasModified(){
		return hasModified;
	}
	
	public void addListener(AbstractionElementListener listener){
		listenersList.add(listener);
	}
	
	public void setSelected(boolean isSelected){
		activeBox.setSelected(isSelected);
	}
	
	public boolean isSelected(){
		return activeBox.isSelected();
	}
	
	public Set<String> getCurrentEncodedActivitySet(){
		return currentEncodedActivitySet;
	}
	
	public String getAbstractionName(){
		abstractionName = abstractionNameTextArea.getText();
		return abstractionName;
	}
}
