package org.processmining.plugins.abstractions.visualization;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.plugins.abstractions.MineAbstractions;
import org.processmining.plugins.abstractions.datatypes.MetricPatternAlphabetCount;
import org.processmining.plugins.abstractions.encoding.TransformLog;
import org.processmining.plugins.abstractions.enumtypes.AbstractionStrategy;
import org.processmining.plugins.abstractions.enumtypes.PatternFilter;
import org.processmining.plugins.abstractions.enumtypes.PatternLengthPreference;
import org.processmining.plugins.abstractions.enumtypes.PatternMetric;
import org.processmining.plugins.abstractions.enumtypes.PatternType;
import org.processmining.plugins.abstractions.enumtypes.RemoveAbstractionStrategy;
import org.processmining.plugins.abstractions.patterns.MinePatterns;
import org.processmining.plugins.abstractions.swingx.ScrollableGridLayout;
import org.processmining.plugins.abstractions.util.UkkonenSuffixTree;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;
import org.processmining.plugins.interactivevisualization.InteractivityManager;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 25 May 2011 (last modified)
 * @since 01 July 2009
 * @version 1.0 
 * @(c)R. P. Jagadeesh Chandra 'JC' Bose, Deparment of Mathematics
 *          and Computer Science, TU/e
 * 
 *  	 Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 */

@SuppressWarnings("serial")
public class PatternAbstractionFrame extends JInternalFrame implements InteractiveVisualization, ScalableComponent {
	UIPluginContext context;
	XLog log;
	
	JPanel overallPanel;
	JPanel parameterSettingsPanel;
	
	JPanel patternTypePanel;
	JPanel patternMetricPanel;
	JPanel patternFilterPanel;
	JPanel abstractionSettingsPanel;
	JPanel setTheorySettingsPanel;
	JPanel resultsPanel;
	JScrollPane patternInfoTableScrollPane;
	
	Set<PatternType> patternTypeSet;
	boolean isSplitTrace;
	int splitTraceThreshold;
	
	Set<PatternMetric> patternMetricSet;
	PatternLengthPreference patternLengthPreference;
	boolean ignoreDuplicateTraces;
	boolean isBasePatterns;
	boolean isPrunePatterns;
	boolean isPatternsDiscovered = false;
	
	Map<PatternFilter, Integer> patternFilterThresholdMap;

	AbstractionStrategy abstractionStrategy;
	boolean isExtendMaximalElements;
	float commonElementsThreshold, diffElementsThreshold;
	int minMaximalElementSizeThreshold;
	JButton findAbstractionsButton;

	PatternInfoTable patternInfoTable;
	JTable table;
	JPanel patternInfoTableControlPanel;
	JLabel alphabetCountLabel;
	Set<TreeSet<String>> deletedPatternAlphabetSet = new HashSet<TreeSet<String>>();
	
	JPanel abstractionResultsPanel;
	JPanel abstractionControlPanel;
	JLabel noAbstractionsLabel;
	
	MinePatterns minePatterns;
	MineAbstractions mineAbstractions;
	
	InteractivityManager interactivityManager;
	boolean logChanged;
	List<UpdateListener> updateListenersList = new ArrayList<UpdateListener>();
	
	public PatternAbstractionFrame(UIPluginContext context, XLog log){
		super("Pattern Abstractions Viewer (R. P. Jagadeesh Chandra 'JC' Bose)");
		this.context = context;
		this.log = log;
		context.getProgress().setCaption("Encoding Log");
		minePatterns = new MinePatterns(this, log);
		overallPanel = SlickerFactory.instance().createRoundedPanel();
		ScrollableGridLayout overallPanelLayout = new ScrollableGridLayout(overallPanel, 2, 1, 0, 0);
		overallPanelLayout.setColumnFixed(1, true);
		
		overallPanel.setLayout(overallPanelLayout);
		
		buildResultsPanel();
		overallPanelLayout.setPosition(resultsPanel, 0, 0);
		overallPanel.add(resultsPanel);
		
		buildParameterSettingsPanel();
		
		JScrollPane parameterSettingsPanelScrollPane = new JScrollPane(parameterSettingsPanel);
		parameterSettingsPanelScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		parameterSettingsPanelScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		overallPanelLayout.setPosition(parameterSettingsPanelScrollPane, 1, 0);
		overallPanel.add(parameterSettingsPanelScrollPane);
		
		getContentPane().add(overallPanel);
		getContentPane().validate();
		getContentPane().repaint();
		pack();

		this.show();
	}
	
	private void buildResultsPanel(){
		resultsPanel = SlickerFactory.instance().createRoundedPanel();
		
		ScrollableGridLayout resultsPanelLayout = new ScrollableGridLayout(resultsPanel, 1, 4, 0, 0);
		resultsPanelLayout.setRowFixed(1, true);
		resultsPanelLayout.setRowFixed(3, true);
		resultsPanel.setLayout(resultsPanelLayout);

		buildAbstractionResultsPanel();
		
		Dimension d = resultsPanel.getPreferredSize();
		JScrollPane abstractionResultsScrollPane = new JScrollPane(abstractionResultsPanel);
		abstractionResultsScrollPane.setPreferredSize(new Dimension(d.width / 2, (int) (0.45 * d.height)));
		abstractionResultsScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		abstractionResultsScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		abstractionResultsScrollPane.getVerticalScrollBar().setBlockIncrement(25);
		abstractionResultsScrollPane.getVerticalScrollBar().setUnitIncrement(25);
		
		resultsPanelLayout.setPosition(abstractionResultsScrollPane, 0, 0);
		resultsPanel.add(abstractionResultsScrollPane);
		
		buildAbstractionControlPanel();
		resultsPanelLayout.setPosition(abstractionControlPanel, 0, 1);
		resultsPanel.add(abstractionControlPanel);
		
		patternInfoTable = new PatternInfoTable(resultsPanel.getBackground());
		table = patternInfoTable.getTable();
		patternInfoTableScrollPane = new JScrollPane(table);
		patternInfoTableScrollPane.setPreferredSize(new Dimension(d.width / 2, (int) (0.45 * d.height)));
		patternInfoTableScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		patternInfoTableScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		resultsPanelLayout.setPosition(patternInfoTableScrollPane, 0, 2);
		resultsPanel.add(patternInfoTableScrollPane);
		
		buildPatternInfoTableControlPanel();
		resultsPanelLayout.setPosition(patternInfoTableControlPanel, 0, 3);
		resultsPanel.add(patternInfoTableControlPanel);
	}
	
	private void buildAbstractionResultsPanel(){
		abstractionResultsPanel = SlickerFactory.instance().createRoundedPanel();
		abstractionResultsPanel.setLayout(new BoxLayout(abstractionResultsPanel, BoxLayout.Y_AXIS));
		mineAbstractions = new MineAbstractions(minePatterns.getEncodedLog(), abstractionResultsPanel);
		//		abstractionResultsPanel.setVisible(false);
	}
	
	private void buildAbstractionControlPanel(){
		abstractionControlPanel = SlickerFactory.instance().createRoundedPanel();
		abstractionControlPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout abstractionControlGBLayout = new GridBagLayout();
		abstractionControlPanel.setLayout(abstractionControlGBLayout);
		
		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		noAbstractionsLabel = SlickerFactory.instance().createLabel("No. Abstractions: 0");
		c.gridx = 0; c.gridy = 0; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(noAbstractionsLabel, c);

		JButton invertSelectionButton = SlickerFactory.instance().createButton("Invert Selection");
		c.gridx = 1; c.gridy = 0; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(invertSelectionButton, c);

		JButton removeAbstractionsButton = SlickerFactory.instance().createButton("Remove Abstractions");
		c.gridx = 2; c.gridy = 0; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(removeAbstractionsButton, c);

		JButton mergeAbstractionsButton = SlickerFactory.instance().createButton("Merge Abstractions");
		c.gridx = 3; c.gridy = 0; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(mergeAbstractionsButton, c);

		final JButton exportSelectedLogButton = SlickerFactory.instance().createButton("Export (Selected) Log");
		c.gridx = 1; c.gridy = 1; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(exportSelectedLogButton, c);

		final JButton visualizeSelectedLogButton = SlickerFactory.instance().createButton("Visualize (Selected) Log");
		c.gridx = 2; c.gridy = 1; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(visualizeSelectedLogButton, c);

		JButton transformLogButton = SlickerFactory.instance().createButton("Transform Log");
		c.gridx = 3; c.gridy = 1; c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		abstractionControlPanel.add(transformLogButton, c);

		invertSelectionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Component[] abstractionResultsPanelComponents = abstractionResultsPanel.getComponents();
				for(Component aec : abstractionResultsPanelComponents){
					if(((AbstractionElementComponent)aec).isSelected()){
						((AbstractionElementComponent)aec).setSelected(false);
					}else{
						((AbstractionElementComponent)aec).setSelected(true);
					}
				}
			}
		});

		removeAbstractionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				RemoveAbstractionsEditDialog removeAbstractionsEditDialog = new RemoveAbstractionsEditDialog(PatternAbstractionFrame.this);
				removeAbstractionsEditDialog.setVisible(true);
//				JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "Remove Abstractions Yet to be Implemented", "Information Message", JOptionPane.INFORMATION_MESSAGE);	
			}
		});
		
		mergeAbstractionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Set<Set<String>> selectedMaximalElementSet = new HashSet<Set<String>>();
				Component[] abstractionResultsPanelComponents = abstractionResultsPanel.getComponents();
				for (Component aec : abstractionResultsPanelComponents) {
					if (((AbstractionElementComponent) aec).isSelected()) {
						selectedMaximalElementSet
							.add(((AbstractionElementComponent) aec)
								.getCurrentEncodedActivitySet());
					}
				}
				mineAbstractions.mergeMaximalElements(selectedMaximalElementSet);
				noAbstractionsLabel.setText("No. Abstractions: "
						+ abstractionResultsPanel.getComponentCount() + " ");
				resultsPanel.revalidate();
				resultsPanel.repaint();
			}
		});
		
		exportSelectedLogButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Set<Set<String>> selectedMaximalElementSet = new HashSet<Set<String>>();
				Component[] abstractionResultsPanelComponents = abstractionResultsPanel.getComponents();
				for(Component aec : abstractionResultsPanelComponents){
					if(((AbstractionElementComponent)aec).isSelected()){
						selectedMaximalElementSet
						.add(((AbstractionElementComponent) aec)
							.getCurrentEncodedActivitySet());
					}
				}
				
				Map<Set<String>,Set<Set<String>>> maximalElementSubsumedPatternAlphabetSetMap = mineAbstractions.getMaximalElementSubsumedPatternAlphabetSetMap();
				Set<Set<String>> selectedPatternAlphabetSet = new HashSet<Set<String>>();
				for(Set<String> maximalElement : selectedMaximalElementSet){
					if(maximalElementSubsumedPatternAlphabetSetMap.containsKey(maximalElement)){
						selectedPatternAlphabetSet.add(maximalElement);
						selectedPatternAlphabetSet.addAll(maximalElementSubsumedPatternAlphabetSetMap.get(maximalElement));
					}
				}

				//Get the set of patterns for the selected alphabets
				Map<TreeSet<String>,TreeSet<String>> patternAlphabetPatternSetMap = minePatterns.getEntireLogCombinedPatternAlphabetPatternSetMap();
				Set<String> selectedPatternAlphabetPatternSet = new HashSet<String>();
				
				for(Set<String> patternAlphabet : selectedPatternAlphabetSet){
					if(patternAlphabetPatternSetMap.containsKey(patternAlphabet))
						selectedPatternAlphabetPatternSet.addAll(patternAlphabetPatternSetMap.get(patternAlphabet));
				}
				
				XLog selectedPatternsLog = minePatterns.getLogSelectedPatterns(selectedPatternAlphabetPatternSet);
				context.getProvidedObjectManager().createProvidedObject("Selected Abstraction Log",
						selectedPatternsLog, XLog.class, context);
				context.getGlobalContext().getResourceManager().getResourceForInstance(selectedPatternsLog).setFavorite(true);
				JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "Log with Traces Containing Selected Patterns is Exported to the Framework", "Information Message", JOptionPane.INFORMATION_MESSAGE);				
			}
		});
		
		visualizeSelectedLogButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "Visualize Selected Log Yet to be Implemented", "Information Message", JOptionPane.INFORMATION_MESSAGE);				
			}
		});
		
		transformLogButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				transformLogButtonActionPerformed();
			}
		});
		
		abstractionControlPanel.setVisible(false);
	}
	
	private void buildPatternInfoTableControlPanel(){
		patternInfoTableControlPanel = SlickerFactory.instance().createRoundedPanel();
		patternInfoTableControlPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout patternInfoTableControlPanelGBLayout = new GridBagLayout();
		patternInfoTableControlPanel.setLayout(patternInfoTableControlPanelGBLayout);

		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);

		alphabetCountLabel = SlickerFactory.instance().createLabel("No. Alphabets: " + 0);
		patternInfoTableControlPanel.add(alphabetCountLabel, c);

		JButton deSelectAllButton = SlickerFactory.instance().createButton("Deselect All");
		c.gridx = 2;
		c.gridy = 0;
		c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		patternInfoTableControlPanel.add(deSelectAllButton, c);

		JButton removeSelectedButton = SlickerFactory.instance().createButton("Remove Selected");
		c.gridx = 3;
		c.gridy = 0;
		c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		patternInfoTableControlPanel.add(removeSelectedButton, c);
	
		JButton visualizeSelectedButton = SlickerFactory.instance().createButton("Visualize Selected");
		c.gridx = 4;
		c.gridy = 0;
		c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		patternInfoTableControlPanel.add(visualizeSelectedButton, c);

		JButton resetTableButton = SlickerFactory.instance().createButton("Reset Table");
		c.gridx = 5;
		c.gridy = 0;
		c.gridwidth = 1;
		c.insets = new Insets(5, 0, 0, 10);
		patternInfoTableControlPanel.add(resetTableButton, c);
		
		patternInfoTableControlPanel.setVisible(false);

		deSelectAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.clearSelection();
			}
		});

		removeSelectedButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println(table.getSelectedRowCount()+" Rows Selected");
				int[] selectedRows = table.getSelectedRows();
				
				for (int i : selectedRows) {
					deletedPatternAlphabetSet.add(patternInfoTable.getFilteredPatternAlphabetList().get(i));
				}
				System.out.println(deletedPatternAlphabetSet);
				preparePatternInfoTable();
			}
		});

		visualizeSelectedButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println(table.getSelectedRowCount()+" Rows Selected");
				JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "Visualize Selected Patterns");			}
		});
		
		resetTableButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				deletedPatternAlphabetSet.clear();
				preparePatternInfoTable();
			}
		});
	}
	
	/**
	 * This method builds the various parameter controls that are used in
	 * defining abstractions
	 */
	private void buildParameterSettingsPanel(){
		parameterSettingsPanel = SlickerFactory.instance().createRoundedPanel();
		
		ScrollableGridLayout parameterSettingsPanelLayout = new ScrollableGridLayout(parameterSettingsPanel, 1, 6, 0, 0);
		parameterSettingsPanelLayout.setRowFixed(0, true);
		parameterSettingsPanelLayout.setColumnFixed(0, true);
		
		parameterSettingsPanel.setLayout(parameterSettingsPanelLayout);
		
		// build the panel with pattern type options
		buildPatternTypePanel();
	
		// build the panel with metrics for pattern counts
		buildPatternMetricPanel();
		
		// build the panel with filtering options for patterns
		buildPatternFilterPanel();
		
		// build the panel with abstraction options
		buildAbstractionSettingsPanel();

		JButton findPatternsButton = SlickerFactory.instance().createButton("Find Patterns");
		JPanel findPatternsButtonPanel = SlickerFactory.instance().createRoundedPanel();
		findPatternsButtonPanel.add(findPatternsButton);
		
		findAbstractionsButton = SlickerFactory.instance().createButton("Find Abstractions");
		JPanel findAbstractionsButtonPanel = SlickerFactory.instance().createRoundedPanel();
		findAbstractionsButtonPanel.add(findAbstractionsButton);
		findAbstractionsButton.setVisible(false);
		
		int yPos = 0;
		parameterSettingsPanelLayout.setPosition(patternTypePanel, 0, yPos);
		parameterSettingsPanel.add(patternTypePanel);
	
		yPos++;
		parameterSettingsPanelLayout.setPosition(patternMetricPanel, 0, yPos);
		parameterSettingsPanel.add(patternMetricPanel);

		yPos++;
		parameterSettingsPanelLayout.setPosition(findPatternsButtonPanel, 0, yPos);
		parameterSettingsPanel.add(findPatternsButtonPanel);
		
		yPos++;
		parameterSettingsPanelLayout.setPosition(patternFilterPanel, 0, yPos);
		parameterSettingsPanel.add(patternFilterPanel);
		
		yPos++;
		parameterSettingsPanelLayout.setPosition(abstractionSettingsPanel, 0, yPos);
		parameterSettingsPanel.add(abstractionSettingsPanel);
	
		yPos++;
		parameterSettingsPanelLayout.setPosition(findAbstractionsButtonPanel, 0, yPos);
		parameterSettingsPanel.add(findAbstractionsButtonPanel);
		
		findPatternsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				deletedPatternAlphabetSet.clear();
				minePatterns.findPatterns(patternTypeSet, patternLengthPreference);
				isPatternsDiscovered = true;
				patternFilterPanel.setVisible(true);
				abstractionSettingsPanel.setVisible(true);
				findAbstractionsButton.setVisible(true);
				System.out.println("Pattern Type Set: "+patternTypeSet);
				preparePatternInfoTable();
				patternInfoTableScrollPane.setVisible(true);
				patternInfoTableControlPanel.setVisible(true);
			}
		});
		
		findAbstractionsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				abstractionResultsPanel.setVisible(true);
				abstractionControlPanel.setVisible(true);
				mineAbstractions.setAbstractionStrategy(abstractionStrategy);
				mineAbstractions.mineSetTheoryAbstractions(patternInfoTable.getFilteredPatternAlphabetList(), isExtendMaximalElements, commonElementsThreshold, minMaximalElementSizeThreshold);
				noAbstractionsLabel.setText("No. Abstractions: "+abstractionResultsPanel.getComponentCount()+" ");
			}
		});
	}
	
	private void buildPatternTypePanel(){
		patternTypePanel = SlickerFactory.instance().createRoundedPanel();
		patternTypePanel.setBorder(BorderFactory.createTitledBorder("Pattern Type"));
		
		ScrollableGridLayout patternTypePanelLayout = new ScrollableGridLayout(patternTypePanel, 1, 5, 0, 0);
		
		patternTypePanelLayout.setRowFixed(0, true);
		patternTypePanelLayout.setRowFixed(1, true);
		patternTypePanelLayout.setRowFixed(2, true);
		patternTypePanelLayout.setRowFixed(3, true);
		patternTypePanelLayout.setRowFixed(4, true);
		
		patternTypePanelLayout.setColumnFixed(0, true);
		patternTypePanel.setLayout(patternTypePanelLayout);
		
		final JCheckBox tandemArrayCheckBox = SlickerFactory.instance().createCheckBox("Tandem Arrays (Loops)", false);
		final JCheckBox maximalRepeatsCheckBox = SlickerFactory.instance().createCheckBox("Maximal Repeats", true);
		final JCheckBox basePatternCheckBox = SlickerFactory.instance().createCheckBox("Base Patterns Only", false);

		patternTypePanelLayout.setPosition(tandemArrayCheckBox, 0, 0);
		patternTypePanel.add(tandemArrayCheckBox);
	
		patternTypePanelLayout.setPosition(maximalRepeatsCheckBox, 0, 1);
		patternTypePanel.add(maximalRepeatsCheckBox);
		
		patternTypePanelLayout.setPosition(basePatternCheckBox, 0, 2);
		patternTypePanel.add(basePatternCheckBox);

		patternTypeSet = new HashSet<PatternType>();
		patternTypeSet.add(PatternType.MaximalRepeats);
		isSplitTrace = false;
		splitTraceThreshold = 500;
		tandemArrayCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(tandemArrayCheckBox.isSelected()){
					patternTypeSet.add(PatternType.TandemArrays);
				}else{
					patternTypeSet.remove(PatternType.TandemArrays);
					if(patternTypeSet.size() == 0){
						JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "At least one pattern type should be selected", "Error Message", JOptionPane.ERROR_MESSAGE);
						tandemArrayCheckBox.setSelected(true);
						patternTypeSet.add(PatternType.TandemArrays);
					}
					
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		maximalRepeatsCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(maximalRepeatsCheckBox.isSelected())
					patternTypeSet.add(PatternType.MaximalRepeats);
				else{
					patternTypeSet.remove(PatternType.MaximalRepeats);
					if(patternTypeSet.size() == 0){
						JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "At least one pattern type should be selected", "Error Message", JOptionPane.ERROR_MESSAGE);
						maximalRepeatsCheckBox.setSelected(true);
						patternTypeSet.add(PatternType.MaximalRepeats);
					}
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		basePatternCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(basePatternCheckBox.isSelected()){
					isBasePatterns = true;
				}else{
					isBasePatterns = false;
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
	}

	private void buildPatternMetricPanel(){
		patternMetricPanel = SlickerFactory.instance().createRoundedPanel();
		patternMetricPanel.setBorder(BorderFactory.createTitledBorder("Pattern Metrics"));
		
		ScrollableGridLayout patternMetricPanelLayout = new ScrollableGridLayout(patternMetricPanel, 1, 5, 0, 0);
		patternMetricPanelLayout.setRowFixed(0, true);
		patternMetricPanelLayout.setRowFixed(1, true);
		patternMetricPanelLayout.setRowFixed(2, true);
		patternMetricPanelLayout.setRowFixed(3, true);
		patternMetricPanelLayout.setRowFixed(4, true);
		
		patternMetricPanel.setLayout(patternMetricPanelLayout);
		
		patternMetricSet = new HashSet<PatternMetric>();
		final JRadioButton nonOverlappingAlphabetCountRadioButton = SlickerFactory.instance().createRadioButton("Non-Overlapping Alphabet Count");
		final JRadioButton overlappingAlphabetCountRadioButton = SlickerFactory.instance().createRadioButton("Overlapping Alphabet Count");
		final JRadioButton nonOverlappingGlobalAlphabetCountRadioButton = SlickerFactory.instance().createRadioButton("Non-Overlapping Global Alphabet Count");
		
		ButtonGroup alphabetCountButtonGroup = new ButtonGroup();
		alphabetCountButtonGroup.add(nonOverlappingAlphabetCountRadioButton);
		alphabetCountButtonGroup.add(overlappingAlphabetCountRadioButton);
		alphabetCountButtonGroup.add(nonOverlappingGlobalAlphabetCountRadioButton);
		
		nonOverlappingAlphabetCountRadioButton.setSelected(true);
		patternMetricSet.add(PatternMetric.NonOverlappingAlphabetCount);
		
		patternMetricPanelLayout.setPosition(nonOverlappingAlphabetCountRadioButton, 0, 0);
		patternMetricPanel.add(nonOverlappingAlphabetCountRadioButton);
		
		patternMetricPanelLayout.setPosition(overlappingAlphabetCountRadioButton, 0, 1);
		patternMetricPanel.add(overlappingAlphabetCountRadioButton);
		
		patternMetricPanelLayout.setPosition(nonOverlappingGlobalAlphabetCountRadioButton, 0, 2);
		patternMetricPanel.add(nonOverlappingGlobalAlphabetCountRadioButton);
		
		final JPanel patternLengthPreferencePanel = SlickerFactory.instance().createRoundedPanel();
		ScrollableGridLayout patternLengthPreferencePanelLayout = new ScrollableGridLayout(patternLengthPreferencePanel, 3, 1, 0, 0);
		
		patternLengthPreferencePanelLayout.setColumnFixed(0, true);
		patternLengthPreferencePanelLayout.setColumnFixed(1, true);
		patternLengthPreferencePanelLayout.setColumnFixed(2, true);
		
		JLabel patternLengthPreferenceLabel = SlickerFactory.instance().createLabel("Pattern Preference ");
		final JRadioButton shorterPatternRadioButton = SlickerFactory.instance().createRadioButton("Shorter");
		final JRadioButton longerPatternRadioButton = SlickerFactory.instance().createRadioButton("Longer");
		shorterPatternRadioButton.setSelected(true);
		patternLengthPreference = PatternLengthPreference.Shorter;
		ButtonGroup patternLengthPreferenceButtonGroup = new ButtonGroup();
		patternLengthPreferenceButtonGroup.add(shorterPatternRadioButton);
		patternLengthPreferenceButtonGroup.add(longerPatternRadioButton);
		
		patternLengthPreferencePanelLayout.setPosition(patternLengthPreferenceLabel, 0, 0);
		patternLengthPreferencePanel.add(patternLengthPreferenceLabel);
		
		patternLengthPreferencePanelLayout.setPosition(shorterPatternRadioButton, 1, 0);
		patternLengthPreferencePanel.add(shorterPatternRadioButton);
		
		patternLengthPreferencePanelLayout.setPosition(longerPatternRadioButton, 2, 0);
		patternLengthPreferencePanel.add(longerPatternRadioButton);
		
		patternLengthPreferencePanel.setVisible(true);
		
		patternMetricPanelLayout.setPosition(patternLengthPreferencePanel, 0, 3);
		patternMetricPanel.add(patternLengthPreferencePanel);
		
		nonOverlappingAlphabetCountRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				patternMetricSet.clear();
				if(nonOverlappingAlphabetCountRadioButton.isSelected()){
					patternMetricSet.add(PatternMetric.NonOverlappingAlphabetCount);
					patternLengthPreferencePanel.setVisible(true);
				}else{
					patternLengthPreferencePanel.setVisible(false);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		overlappingAlphabetCountRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				patternMetricSet.clear();
				if(overlappingAlphabetCountRadioButton.isSelected()){
					patternMetricSet.add(PatternMetric.OverlappingAlphabetCount);
					patternLengthPreferencePanel.setVisible(false);
					if(isPatternsDiscovered)
						preparePatternInfoTable();
				}
			}
		});
		
		nonOverlappingGlobalAlphabetCountRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				patternMetricSet.clear();
				if(nonOverlappingGlobalAlphabetCountRadioButton.isSelected()){
					patternLengthPreferencePanel.setVisible(true);
					patternMetricSet.add(PatternMetric.NonOverlappingGlobalAlphabetCount);
				}else{
					patternLengthPreferencePanel.setVisible(false);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
	
		shorterPatternRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(shorterPatternRadioButton.isSelected()){
					patternLengthPreference = PatternLengthPreference.Shorter;
					minePatterns.setPatternLengthPreference(patternLengthPreference);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		longerPatternRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(longerPatternRadioButton.isSelected()){
					patternLengthPreference = PatternLengthPreference.Longer;
					minePatterns.setPatternLengthPreference(patternLengthPreference);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
	}

	private void buildPatternFilterPanel(){
		patternFilterPanel = SlickerFactory.instance().createRoundedPanel();
		patternFilterPanel.setBorder(BorderFactory.createTitledBorder("Pattern Filters"));
		
		ScrollableGridLayout patternFilterPanelLayout = new ScrollableGridLayout(patternFilterPanel, 3, 6, 0, 0);
		patternFilterPanelLayout.setRowFixed(0, true);
		patternFilterPanelLayout.setRowFixed(1, true);
		patternFilterPanelLayout.setRowFixed(2, true);
		patternFilterPanelLayout.setRowFixed(3, true);
		patternFilterPanelLayout.setRowFixed(4, true);
		patternFilterPanelLayout.setRowFixed(5, true);
		
		patternFilterPanelLayout.setColumnFixed(0, true);
		patternFilterPanelLayout.setColumnFixed(1, true);
		patternFilterPanelLayout.setColumnFixed(2, true);
		
		patternFilterPanel.setLayout(patternFilterPanelLayout);
		
		
		final JCheckBox ignoreDuplicateTracesCheckBox = SlickerFactory.instance().createCheckBox("Ignore Duplicate Traces", true);
		ignoreDuplicateTraces = true;
		
		final JCheckBox prunePatternsCheckBox = SlickerFactory.instance().createCheckBox("Prune Patterns", true);
		isPrunePatterns = true;
		
		final JCheckBox frequencyFilterCheckBox = SlickerFactory.instance().createCheckBox("Frequency", false);
		final JCheckBox instancePercentageFilterCheckBox = SlickerFactory.instance().createCheckBox("Instance Count %", false);
		final JCheckBox alphabetSizeFilterCheckBox = SlickerFactory.instance().createCheckBox("Alphabet Size", false);
		final JCheckBox conservednessFilterCheckBox = SlickerFactory.instance().createCheckBox("Conservedness", false);

		Dimension sliderPreferredSize = new Dimension(100, 10);
		final JSlider frequencyFilterSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		frequencyFilterSlider.setMinimum(0);
		frequencyFilterSlider.setMaximum(100);
		frequencyFilterSlider.setValue(10);
		frequencyFilterSlider.setEnabled(false);
		frequencyFilterSlider.setPreferredSize(sliderPreferredSize);
		final JLabel frequencyFilterThresholdLabel = SlickerFactory.instance().createLabel("10");
		
		final JSlider instanceCountPercentageFilterSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		instanceCountPercentageFilterSlider.setMinimum(0);
		instanceCountPercentageFilterSlider.setMaximum(100);
		instanceCountPercentageFilterSlider.setValue(10);
		instanceCountPercentageFilterSlider.setEnabled(false);
		instanceCountPercentageFilterSlider.setPreferredSize(sliderPreferredSize);
		final JLabel instanceCountPercentageFilterThresholdLabel = SlickerFactory.instance().createLabel("10");
		
		final JSlider alphabetSizeFilterSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		alphabetSizeFilterSlider.setMinimum(1);
		alphabetSizeFilterSlider.setMaximum(10);
		alphabetSizeFilterSlider.setValue(1);
		alphabetSizeFilterSlider.setEnabled(false);
		alphabetSizeFilterSlider.setPreferredSize(sliderPreferredSize);
		final JLabel alphabetSizeFilterThresholdLabel = SlickerFactory.instance().createLabel("1");
		
		final JSlider conservednessFilterSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		conservednessFilterSlider.setMinimum(0);
		conservednessFilterSlider.setMaximum(100);
		conservednessFilterSlider.setValue(50);
		conservednessFilterSlider.setEnabled(false);
		conservednessFilterSlider.setPreferredSize(sliderPreferredSize);
		final JLabel conservednessFilterThresholdLabel = SlickerFactory.instance().createLabel("50");
		
		int yPos = 0;
		patternFilterPanelLayout.setPosition(ignoreDuplicateTracesCheckBox, 0, yPos);
		patternFilterPanel.add(ignoreDuplicateTracesCheckBox);
		
		yPos++;
		patternFilterPanelLayout.setPosition(prunePatternsCheckBox, 0, yPos);
		patternFilterPanel.add(prunePatternsCheckBox);
		
		yPos++;
		patternFilterPanelLayout.setPosition(frequencyFilterCheckBox, 0, yPos);
		patternFilterPanel.add(frequencyFilterCheckBox);
		patternFilterPanelLayout.setPosition(frequencyFilterSlider, 1, yPos);
		patternFilterPanel.add(frequencyFilterSlider);
		patternFilterPanelLayout.setPosition(frequencyFilterThresholdLabel, 2, yPos);
		patternFilterPanel.add(frequencyFilterThresholdLabel);
		
		yPos++;
		patternFilterPanelLayout.setPosition(instancePercentageFilterCheckBox, 0, yPos);
		patternFilterPanel.add(instancePercentageFilterCheckBox);
		patternFilterPanelLayout.setPosition(instanceCountPercentageFilterSlider, 1, yPos);
		patternFilterPanel.add(instanceCountPercentageFilterSlider);
		patternFilterPanelLayout.setPosition(instanceCountPercentageFilterThresholdLabel, 2, yPos);
		patternFilterPanel.add(instanceCountPercentageFilterThresholdLabel);
		
		yPos++;
		patternFilterPanelLayout.setPosition(alphabetSizeFilterCheckBox, 0, yPos);
		patternFilterPanel.add(alphabetSizeFilterCheckBox);
		patternFilterPanelLayout.setPosition(alphabetSizeFilterSlider, 1, yPos);
		patternFilterPanel.add(alphabetSizeFilterSlider);
		patternFilterPanelLayout.setPosition(alphabetSizeFilterThresholdLabel, 2, yPos);
		patternFilterPanel.add(alphabetSizeFilterThresholdLabel);
		
		yPos++;
		patternFilterPanelLayout.setPosition(conservednessFilterCheckBox, 0, yPos);
		patternFilterPanel.add(conservednessFilterCheckBox);
		patternFilterPanelLayout.setPosition(conservednessFilterSlider, 1, yPos);
		patternFilterPanel.add(conservednessFilterSlider);
		patternFilterPanelLayout.setPosition(conservednessFilterThresholdLabel, 2, yPos);
		patternFilterPanel.add(conservednessFilterThresholdLabel);
		
		ignoreDuplicateTracesCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(ignoreDuplicateTracesCheckBox.isSelected())
					ignoreDuplicateTraces = true;
				else
					ignoreDuplicateTraces = false;
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		prunePatternsCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(prunePatternsCheckBox.isSelected())
					isPrunePatterns = true;
				else
					isPrunePatterns = false;
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		patternFilterThresholdMap = new HashMap<PatternFilter, Integer>();
		frequencyFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(frequencyFilterCheckBox.isSelected()){
					frequencyFilterSlider.setEnabled(true);
					patternFilterThresholdMap.put(PatternFilter.Frequency, new Integer(frequencyFilterSlider.getValue()));
				}else{
					frequencyFilterSlider.setEnabled(false);
					if(patternFilterThresholdMap.containsKey(PatternFilter.Frequency))
						patternFilterThresholdMap.remove(PatternFilter.Frequency);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		instancePercentageFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(instancePercentageFilterCheckBox.isSelected()){
					instanceCountPercentageFilterSlider.setEnabled(true);
					patternFilterThresholdMap.put(PatternFilter.InstanceCountPercentage, new Integer(instanceCountPercentageFilterSlider.getValue()));
				}else{
					instanceCountPercentageFilterSlider.setEnabled(false);
					if(patternFilterThresholdMap.containsKey(PatternFilter.InstanceCountPercentage))
						patternFilterThresholdMap.remove(PatternFilter.InstanceCountPercentage);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		alphabetSizeFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(alphabetSizeFilterCheckBox.isSelected()){
					alphabetSizeFilterSlider.setEnabled(true);
					patternFilterThresholdMap.put(PatternFilter.AlphabetSize, new Integer(alphabetSizeFilterSlider.getValue()));
				}else{
					alphabetSizeFilterSlider.setEnabled(false);
					if(patternFilterThresholdMap.containsKey(PatternFilter.AlphabetSize))
						patternFilterThresholdMap.remove(PatternFilter.AlphabetSize);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		conservednessFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(conservednessFilterCheckBox.isSelected()){
					conservednessFilterSlider.setEnabled(true);
					patternFilterThresholdMap.put(PatternFilter.Conservedness, new Integer(conservednessFilterSlider.getValue()));
				}else{
					conservednessFilterSlider.setEnabled(false);
					if(patternFilterThresholdMap.containsKey(PatternFilter.Conservedness))
						patternFilterThresholdMap.remove(PatternFilter.Conservedness);
				}
				if(isPatternsDiscovered)
					preparePatternInfoTable();
			}
		});
		
		frequencyFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!frequencyFilterSlider.getValueIsAdjusting()){
					patternFilterThresholdMap.put(PatternFilter.Frequency, new Integer(frequencyFilterSlider.getValue()));
					frequencyFilterThresholdLabel.setText(frequencyFilterSlider.getValue()+"");
					if(isPatternsDiscovered)
						preparePatternInfoTable();
				}
			}
		});
		
		instanceCountPercentageFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!instanceCountPercentageFilterSlider.getValueIsAdjusting()){
					patternFilterThresholdMap.put(PatternFilter.InstanceCountPercentage, new Integer(instanceCountPercentageFilterSlider.getValue()));
					instanceCountPercentageFilterThresholdLabel.setText(instanceCountPercentageFilterSlider.getValue()+"");
					if(isPatternsDiscovered)
						preparePatternInfoTable();
				}
			}
		});
		
		alphabetSizeFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!alphabetSizeFilterSlider.getValueIsAdjusting()){
					patternFilterThresholdMap.put(PatternFilter.AlphabetSize, new Integer(alphabetSizeFilterSlider.getValue()));
					alphabetSizeFilterThresholdLabel.setText(alphabetSizeFilterSlider.getValue()+"");
					if(isPatternsDiscovered)
						preparePatternInfoTable();
				}
			}
		});
		
		conservednessFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!conservednessFilterSlider.getValueIsAdjusting()){
					patternFilterThresholdMap.put(PatternFilter.Conservedness, new Integer(conservednessFilterSlider.getValue()));
					conservednessFilterThresholdLabel.setText(conservednessFilterSlider.getValue()+"");
					if(isPatternsDiscovered)
						preparePatternInfoTable();
				}
			}
		});
		
		patternFilterPanel.setVisible(false);
	}

	private void buildAbstractionSettingsPanel(){
		abstractionSettingsPanel = SlickerFactory.instance().createRoundedPanel();
		abstractionSettingsPanel.setBorder(BorderFactory.createTitledBorder("Abstraction Settings"));

		ScrollableGridLayout abstractionSettingsPanelLayout = new ScrollableGridLayout(abstractionSettingsPanel, 1, 4, 0, 0);
		abstractionSettingsPanelLayout.setRowFixed(0, true);
		abstractionSettingsPanelLayout.setRowFixed(1, true);
		abstractionSettingsPanelLayout.setRowFixed(2, true);
		abstractionSettingsPanelLayout.setRowFixed(3, true);
		
		abstractionSettingsPanel.setLayout(abstractionSettingsPanelLayout);
		
		final JRadioButton setTheoryRadioButton = SlickerFactory.instance().createRadioButton("Set Theory");
		final JRadioButton graphTheoryRadioButton = SlickerFactory.instance().createRadioButton("Graph Theory");
		
		ButtonGroup abstractionStrategyButtonGroup = new ButtonGroup();
		abstractionStrategyButtonGroup.add(setTheoryRadioButton);
		abstractionStrategyButtonGroup.add(graphTheoryRadioButton);
		
		setTheoryRadioButton.setSelected(true);
		abstractionStrategy = AbstractionStrategy.SetTheory;
		
		buildSetTheorySettingsPanel();
		
		int yPos = 0;
		abstractionSettingsPanelLayout.setPosition(setTheoryRadioButton, 0, yPos);
		abstractionSettingsPanel.add(setTheoryRadioButton);
		
		yPos++;
		abstractionSettingsPanelLayout.setPosition(graphTheoryRadioButton, 0, yPos);
		abstractionSettingsPanel.add(graphTheoryRadioButton);
		
		yPos++;
		abstractionSettingsPanelLayout.setPosition(setTheorySettingsPanel, 0, yPos);
		abstractionSettingsPanel.add(setTheorySettingsPanel);
	
		setTheoryRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(setTheoryRadioButton.isSelected()){
					abstractionStrategy = AbstractionStrategy.SetTheory;
					setTheorySettingsPanel.setVisible(true);
				}else{
					setTheorySettingsPanel.setVisible(false);
				}
			}
		});	
		
		graphTheoryRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(graphTheoryRadioButton.isSelected()){
					setTheorySettingsPanel.setVisible(false);
					JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "Graph Theory Based Abstraction yet to be implemented", "Invalid Option", JOptionPane.ERROR_MESSAGE);
				}else{
					setTheorySettingsPanel.setVisible(true);
				}
			}
		});
		
		abstractionSettingsPanel.setVisible(false);
	}
	
	private void buildSetTheorySettingsPanel(){
		setTheorySettingsPanel = SlickerFactory.instance().createRoundedPanel();
		setTheorySettingsPanel.setBorder(BorderFactory.createTitledBorder("Merge Maximal Elements"));
		
		ScrollableGridLayout setTheorySettingsPanelLayout = new ScrollableGridLayout(setTheorySettingsPanel, 3, 3, 0, 0);
	
		setTheorySettingsPanelLayout.setColumnFixed(0, true);
		setTheorySettingsPanelLayout.setColumnFixed(1, true);
		setTheorySettingsPanelLayout.setColumnFixed(2, true);
		
		setTheorySettingsPanelLayout.setRowFixed(0, true);
		setTheorySettingsPanelLayout.setRowFixed(1, true);
		setTheorySettingsPanelLayout.setRowFixed(2, true);
		
		setTheorySettingsPanel.setLayout(setTheorySettingsPanelLayout);
		
		final JCheckBox commonElementsThresholdCheckBox = SlickerFactory.instance().createCheckBox("% Common Elements", false);
		final JCheckBox diffElementsThresholdCheckBox = SlickerFactory.instance().createCheckBox("% Diff Elements", true);
		final JCheckBox minMaximalElementSizeThresholdCheckBox = SlickerFactory.instance().createCheckBox("Min. Max Element Size", false);
		
		Dimension preferredSliderSize = new Dimension(100,10);
		final JSlider commonElementsThresholdSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		commonElementsThresholdSlider.setPreferredSize(preferredSliderSize);
		commonElementsThresholdSlider.setMinimum(0);
		commonElementsThresholdSlider.setMaximum(100);
		commonElementsThresholdSlider.setValue(100);
		commonElementsThresholdSlider.setEnabled(true);
		commonElementsThreshold = 100;
		final JLabel commonElementsThresholdLabel = SlickerFactory.instance().createLabel("100");
		
		final JSlider diffElementsThresholdSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		diffElementsThresholdSlider.setPreferredSize(preferredSliderSize);
		diffElementsThresholdSlider.setMinimum(0);
		diffElementsThresholdSlider.setMaximum(100);
		diffElementsThresholdSlider.setValue(0);
		diffElementsThresholdSlider.setEnabled(true);
		final JLabel diffElementsThresholdLabel = SlickerFactory.instance().createLabel("0");

		final JSlider minMaximalElementSizeThresholdSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		minMaximalElementSizeThresholdSlider.setPreferredSize(preferredSliderSize);
		minMaximalElementSizeThresholdSlider.setMinimum(1);
		minMaximalElementSizeThresholdSlider.setMaximum(20);
		minMaximalElementSizeThresholdSlider.setValue(1);
		minMaximalElementSizeThresholdSlider.setEnabled(true);
		minMaximalElementSizeThreshold = 1;
		final JLabel minMaximalElementSizeThresholdLabel = SlickerFactory.instance().createLabel("1");
	
		isExtendMaximalElements = false;
		
		int yPos = 0;
		setTheorySettingsPanelLayout.setPosition(commonElementsThresholdCheckBox, 0, yPos);
		setTheorySettingsPanel.add(commonElementsThresholdCheckBox);
		setTheorySettingsPanelLayout.setPosition(commonElementsThresholdSlider, 1, yPos);
		setTheorySettingsPanel.add(commonElementsThresholdSlider);
		setTheorySettingsPanelLayout.setPosition(commonElementsThresholdLabel, 2, yPos);
		setTheorySettingsPanel.add(commonElementsThresholdLabel);
		
//		yPos++;
//		setTheorySettingsPanelLayout.setPosition(diffElementsThresholdCheckBox, 0, yPos);
//		setTheorySettingsPanel.add(diffElementsThresholdCheckBox);
//		setTheorySettingsPanelLayout.setPosition(diffElementsThresholdSlider, 1, yPos);
//		setTheorySettingsPanel.add(diffElementsThresholdSlider);
//		setTheorySettingsPanelLayout.setPosition(diffElementsThresholdLabel, 2, yPos);
//		setTheorySettingsPanel.add(diffElementsThresholdLabel);
	
		yPos++;
		setTheorySettingsPanelLayout.setPosition(minMaximalElementSizeThresholdCheckBox, 0, yPos);
		setTheorySettingsPanel.add(minMaximalElementSizeThresholdCheckBox);
		setTheorySettingsPanelLayout.setPosition(minMaximalElementSizeThresholdSlider, 1, yPos);
		setTheorySettingsPanel.add(minMaximalElementSizeThresholdSlider);
		setTheorySettingsPanelLayout.setPosition(minMaximalElementSizeThresholdLabel, 2, yPos);
		setTheorySettingsPanel.add(minMaximalElementSizeThresholdLabel);
		
		commonElementsThresholdCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(commonElementsThresholdCheckBox.isSelected()){
					commonElementsThresholdSlider.setEnabled(true);
					commonElementsThresholdLabel.setText(commonElementsThresholdSlider.getValue()+"");
					isExtendMaximalElements = true;
					commonElementsThreshold = commonElementsThresholdSlider.getValue()/100.0f;
				}else{
					commonElementsThresholdSlider.setEnabled(false);
					if(!diffElementsThresholdCheckBox.isSelected())
						isExtendMaximalElements = false;
				}
			}
		});
		
		diffElementsThresholdCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(diffElementsThresholdCheckBox.isSelected()){
					diffElementsThresholdSlider.setEnabled(true);
					diffElementsThresholdLabel.setText(diffElementsThresholdSlider.getValue()+"");
					isExtendMaximalElements = true;
					diffElementsThreshold = diffElementsThresholdSlider.getValue()/100.0f;
				}else{
					diffElementsThresholdSlider.setEnabled(false);
					if(!commonElementsThresholdCheckBox.isSelected())
						isExtendMaximalElements = false;
				}
			}
		});
		
		minMaximalElementSizeThresholdCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(minMaximalElementSizeThresholdCheckBox.isSelected()){
					minMaximalElementSizeThresholdSlider.setEnabled(true);
					minMaximalElementSizeThresholdLabel.setText(minMaximalElementSizeThresholdSlider.getValue()+"");
					isExtendMaximalElements = true;
					minMaximalElementSizeThreshold = minMaximalElementSizeThresholdSlider.getValue();
				}else{
					minMaximalElementSizeThresholdSlider.setEnabled(false);
					if(!commonElementsThresholdCheckBox.isSelected())
						isExtendMaximalElements = false;
				}
			}
		});

		commonElementsThresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!commonElementsThresholdSlider.getValueIsAdjusting()){
					commonElementsThreshold = commonElementsThresholdSlider.getValue();
					commonElementsThresholdLabel.setText(commonElementsThresholdSlider.getValue()+"");
				}
			}
		});
		
		diffElementsThresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!diffElementsThresholdSlider.getValueIsAdjusting()){
					diffElementsThreshold = diffElementsThresholdSlider.getValue();
					diffElementsThresholdLabel.setText(diffElementsThresholdSlider.getValue()+"");
				}
			}
		});
		
		minMaximalElementSizeThresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(!minMaximalElementSizeThresholdSlider.getValueIsAdjusting()){
					minMaximalElementSizeThreshold = minMaximalElementSizeThresholdSlider.getValue();
					minMaximalElementSizeThresholdLabel.setText(minMaximalElementSizeThreshold+"");
				}
			}
		});
	}
	
	private void preparePatternInfoTable(){
		MetricPatternAlphabetCount metricPatternAlphabetCount;
		
		if(patternMetricSet.contains(PatternMetric.NonOverlappingAlphabetCount)){
			if(patternLengthPreference == PatternLengthPreference.Shorter)
				metricPatternAlphabetCount = minePatterns.getNonOverlappingShorterPatternAlphabetCount();
			else
				metricPatternAlphabetCount = minePatterns.getNonOverlappingLongerPatternAlphabetCount();
		}else if(patternMetricSet.contains(PatternMetric.OverlappingAlphabetCount)){
			metricPatternAlphabetCount = minePatterns.getOverlappingPatternAlphabetCount();
		}else if(patternMetricSet.contains(PatternMetric.NonOverlappingGlobalAlphabetCount)){
			if(patternLengthPreference == PatternLengthPreference.Shorter)
				metricPatternAlphabetCount = minePatterns.getNonOverlappingGlobalShorterPatternAlphabetCount();
			else
				metricPatternAlphabetCount = minePatterns.getNonOverlappingGlobalLongerPatternAlphabetCount();
		}else{
			metricPatternAlphabetCount = null;
			JOptionPane.showMessageDialog(this, "Invalid Selections. No Pattern Metric is Chosen", "Error Message", JOptionPane.ERROR_MESSAGE);
		}

		patternInfoTable.prepareData(minePatterns.getEncodedLog(), deletedPatternAlphabetSet, metricPatternAlphabetCount, minePatterns.getConservednessMetric(), patternTypeSet, isBasePatterns, ignoreDuplicateTraces, patternFilterThresholdMap);
		table = patternInfoTable.getTable();
		alphabetCountLabel.setText("No. Alphabets: "+table.getRowCount()+" ");
	}
	
	public UIPluginContext getContext(){
		return context;
	}
	
	public void removeAbstractions(RemoveAbstractionStrategy removeAbstractionStrategy) {
		if(removeAbstractionStrategy == RemoveAbstractionStrategy.NONE)
			return;
		
		Set<Set<String>> selectedMaximalElementSet = new HashSet<Set<String>>();
		Component[] abstractionResultsPanelComponents = abstractionResultsPanel
				.getComponents();
		for (Component aec : abstractionResultsPanelComponents) {
			if (((AbstractionElementComponent) aec).isSelected()) {
				selectedMaximalElementSet
						.add(((AbstractionElementComponent) aec)
								.getCurrentEncodedActivitySet());
			}
		}
		mineAbstractions.removeMaximalElements(selectedMaximalElementSet, removeAbstractionStrategy);
		noAbstractionsLabel.setText("No. Abstractions: "
				+ abstractionResultsPanel.getComponentCount() + " ");
		resultsPanel.revalidate();
		resultsPanel.repaint();
	}
	
	private Set<String> getAIIPSet(Set<Set<String>> patternAlphabetSet){
		Set<String> activitiesInvolvedInPatternSet = new HashSet<String>();
		for(Set<String> patternAlphabet : patternAlphabetSet)
			activitiesInvolvedInPatternSet.addAll(patternAlphabet);
		return activitiesInvolvedInPatternSet;
	}
	
	private void transformLogButtonActionPerformed(){
		String transformLogAbsoluteFileName = null;
		Set<Set<String>> selectedMaximalElementSet = new HashSet<Set<String>>();
		Component[] abstractionElementComponentList = abstractionResultsPanel.getComponents();
		System.out.println("No. Components: "+abstractionElementComponentList.length);
		for(Component aec : abstractionElementComponentList){
			if(aec instanceof AbstractionElementComponent && ((AbstractionElementComponent)aec).isSelected()){
				System.out.println("Abs Selected: "+((AbstractionElementComponent)aec).getAbstractionName());
				selectedMaximalElementSet.add(((AbstractionElementComponent)aec).currentEncodedActivitySet);
			}
		}
		
		Set<String> abstractionNameSet = new HashSet<String>();
		for(Component aec : abstractionElementComponentList){
			if(aec instanceof AbstractionElementComponent && ((AbstractionElementComponent)aec).isSelected()){
				if(((AbstractionElementComponent)aec).currentEncodedActivitySet.size() > 1)
					abstractionNameSet.add(((AbstractionElementComponent)aec).getAbstractionName().replaceAll("\\[", "").replaceAll("\\]", ""));
			}
		}
		
		if (selectedMaximalElementSet.size() == 0) {
			JOptionPane.showMessageDialog(this,	"Select one or more abstractions for transforming log", "Error Message", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		Set<Set<String>> selectedMaximalElementSubsumedPatternAlphabetSet = new HashSet<Set<String>>();
		Map<Set<String>, Set<Set<String>>> mergedMaximalElementSubsumedMaximalElementSetMap = mineAbstractions.getMergedMaximalElementSubsumedMaximalElementSetMap();
		for(Set<String> selectedMaximalElement : selectedMaximalElementSet){
			if(mergedMaximalElementSubsumedMaximalElementSetMap.containsKey(selectedMaximalElement)){
				selectedMaximalElementSubsumedPatternAlphabetSet.addAll(mergedMaximalElementSubsumedMaximalElementSetMap.get(selectedMaximalElement));
			}
		}
		
		selectedMaximalElementSet.addAll(selectedMaximalElementSubsumedPatternAlphabetSet);
		
		Map<Set<String>, Set<Set<String>>> maximalElementSubsumedPatternAlphabetSetMap = mineAbstractions.getMaximalElementSubsumedPatternAlphabetSetMap();
		if(maximalElementSubsumedPatternAlphabetSetMap != null){
			for(Set<String> selectedMaximalElement : selectedMaximalElementSet){
				if(maximalElementSubsumedPatternAlphabetSetMap.containsKey(selectedMaximalElement))
					selectedMaximalElementSubsumedPatternAlphabetSet.addAll(maximalElementSubsumedPatternAlphabetSetMap.get(selectedMaximalElement));
			}
		}
		
		JFileChooser fileChooser = new JFileChooser(".");
		int returnValue = fileChooser.showSaveDialog(PatternAbstractionFrame.this);
		if (returnValue == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			if (file.exists()) {
				int response = JOptionPane.showConfirmDialog(PatternAbstractionFrame.this,
						"Overwrite existing file?", "Confirm overwrite", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE);
				if (response == JOptionPane.CANCEL_OPTION) {
					return;
				}
			}
			transformLogAbsoluteFileName = file.getAbsolutePath();
		} else if (returnValue == JFileChooser.CANCEL_OPTION) {
			return;
		}
		
		boolean considerFilteredPatterns = false;
//		int response = JOptionPane.showConfirmDialog(PatternAbstractionFrame.this,
//				"Do you want to consider filtered patterns (on alphabet size) for abstractions?", "Confirm Consideration", JOptionPane.YES_NO_OPTION,
//				JOptionPane.QUESTION_MESSAGE);
//		if (response == JOptionPane.YES_OPTION) {
//			considerFilteredPatterns = true;
//		}
		
		System.out.println("Selected ME Set: "+selectedMaximalElementSet);
		System.out.println("Selected ME-Subsumed Set: "+selectedMaximalElementSubsumedPatternAlphabetSet);
		
		Map<Set<String>, TreeSet<String>> filteredPatternAlphabetPatternSetMap = mineAbstractions.processPatternModifications(patternInfoTable.getPatternAlphabetPatternSetMap());
		//Filter those not selected and those not subsumed by the selected maximal elements
		Set<Set<String>> nonSelectedMaximalElementSet = new HashSet<Set<String>>();
		for(Set<String> filteredPatternAlphabet : filteredPatternAlphabetPatternSetMap.keySet()){
			if(!selectedMaximalElementSet.contains(filteredPatternAlphabet) && !selectedMaximalElementSubsumedPatternAlphabetSet.contains(filteredPatternAlphabet)){
				nonSelectedMaximalElementSet.add(filteredPatternAlphabet);
			}
		}
		
		
		for(Set<String> nonSelectedMaximalElement : nonSelectedMaximalElementSet)
			filteredPatternAlphabetPatternSetMap.remove(nonSelectedMaximalElement);
	
		System.out.println("Filtered Pattenr Alphabet Set: "+filteredPatternAlphabetPatternSetMap.keySet());
		
		Set<String> decodedPatternAlphabet = new HashSet<String>();
		Map<String, String> charActivityMap = minePatterns.getEncodedLog().getCharActivityMap();
		Set<String> allActivitiesInvolvedInMaximalElementSet = new HashSet<String>();
		for(Set<String> selectedMaximalElement : selectedMaximalElementSet){
			allActivitiesInvolvedInMaximalElementSet.addAll(selectedMaximalElement);
			decodedPatternAlphabet.clear();
			for(String encodedActivity : selectedMaximalElement)
				decodedPatternAlphabet.add(charActivityMap.get(encodedActivity));
		}
		
		
		
		if(considerFilteredPatterns){
			Map<TreeSet<String>, TreeSet<String>> originalPatternAlphabetPatternSetMap = null;
			if(patternTypeSet.size() > 1 && isBasePatterns){
				originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogCombinedBasePatternAlphabetPatternSetMap();
			}else if(patternTypeSet.size() > 1 && !isBasePatterns){
				originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogCombinedPatternAlphabetPatternSetMap();
			}else if(patternTypeSet.size() == 1 && isBasePatterns){
				if(patternTypeSet.iterator().next().equals(PatternType.TandemArrays))
					originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogTandemArrayBasePatternAlphabetPatternSetMap();
				else if(patternTypeSet.iterator().next().equals(PatternType.MaximalRepeats))
					originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogMaximalRepeatBasePatternAlphabetPatternSetMap();
			}else if(patternTypeSet.size() == 1 && !isBasePatterns){
				if(patternTypeSet.iterator().next().equals(PatternType.TandemArrays))
					originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogTandemArrayPatternAlphabetPatternSetMap();
				else if(patternTypeSet.iterator().next().equals(PatternType.MaximalRepeats))
					originalPatternAlphabetPatternSetMap = minePatterns.getEntireLogMaximalRepeatPatternAlphabetPatternSetMap();
			}
	
			if(originalPatternAlphabetPatternSetMap != null){
				for(Set<String> patternAlphabet : originalPatternAlphabetPatternSetMap.keySet()){
					decodedPatternAlphabet.clear();
					for(String encodedActivity : patternAlphabet)
						decodedPatternAlphabet.add(charActivityMap.get(encodedActivity));
					
					for(Set<String> selectedMaximalElement : selectedMaximalElementSet){
						if(selectedMaximalElement.containsAll(patternAlphabet)){
							filteredPatternAlphabetPatternSetMap.put(patternAlphabet, originalPatternAlphabetPatternSetMap.get(patternAlphabet));
							break;
						}
					}
				}
			}
		}
		
		Set<String> aIIPSet = getAIIPSet(filteredPatternAlphabetPatternSetMap.keySet());
		
		Set<String> aNIIPSet = new HashSet<String>();
		aNIIPSet.addAll(minePatterns.getEncodedLog().getCharActivityMap().keySet());
		aNIIPSet.removeAll(aIIPSet);
		
		Set<String> dANIIPSet = minePatterns.getDecodedAlphabet(aNIIPSet);
		Set<String> sNIASet = new HashSet<String>();
		
		System.out.println("Activities not involved in abs:"+dANIIPSet);
		if(dANIIPSet.size()  > 0){
			final JList aNIIPList = new JList(dANIIPSet.toArray());
			JScrollPane scrollPane = new JScrollPane(aNIIPList);
			final JLabel label = new JLabel("<HTML>Please select the interesting activities that you would like to retain from the list below </HTML>");
			 
			final JOptionPane optionPane =
			      new JOptionPane(new Object[] { label, scrollPane },
			            JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION) {
			 
			        @Override
			        public void selectInitialValue() {
			          super.selectInitialValue();
			          aNIIPList.requestFocus();
			        }
			      };
			      
			final JDialog dialog = optionPane.createDialog(null, "Activities Not Involved in Any Pattern");
			dialog.setVisible(true);
			final int result = ((Integer) optionPane.getValue()).intValue();
			System.out.println(JOptionPane.OK_OPTION == result ? "Selected: " : "User canceled!");
			
			if(result == JOptionPane.OK_OPTION){
				Object[] selectedValues = aNIIPList.getSelectedValues();
				for(Object selectedValue : selectedValues)
					sNIASet.add((String)selectedValue);
			}else{
				JOptionPane.showMessageDialog(PatternAbstractionFrame.this, "User Cancelled. Quitting Transform Log", "Cancel Message", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		
		Set<String> sNIEASet = minePatterns.getEncodedAlphabet(sNIASet);
		
		TransformLog transformLog = new TransformLog(log, minePatterns.getEncodedLog(), transformLogAbsoluteFileName);
		transformLog.transform(filteredPatternAlphabetPatternSetMap, getAbstractionNames(filteredPatternAlphabetPatternSetMap), sNIEASet);
	}
	
	public Map<Set<String>, Set<String>> getAbstractionNames(Map<Set<String>, TreeSet<String>> filteredPatternAlphabetPatternSetMap){
		Map<Set<String>, Set<String>> filteredPatternAlphabetAbstractionNameSetMap = new HashMap<Set<String>, Set<String>>();
		Component[] abstractionElementComponents = abstractionResultsPanel.getComponents();
		
		Map<Set<String>,String> maximalElementAbstractionNameMap = new HashMap<Set<String>, String>();
		for(Component aec : abstractionElementComponents){
			if(((AbstractionElementComponent)aec).isSelected())
				maximalElementAbstractionNameMap.put(((AbstractionElementComponent)aec).currentEncodedActivitySet, ((AbstractionElementComponent)aec).abstractionName);
		}
		
		for(Set<String> maximalElement : maximalElementAbstractionNameMap.keySet())
			System.out.println(maximalElement+" @ "+maximalElementAbstractionNameMap.get(maximalElement));
		
		Set<String> filteredPatternAlphabetAbstractionNameSet;
		for(Set<String> filteredPatternAlphabet : filteredPatternAlphabetPatternSetMap.keySet()){
			filteredPatternAlphabetAbstractionNameSet = new HashSet<String>();
			for(Set<String> maximalElement : maximalElementAbstractionNameMap.keySet()){
				if(maximalElement.containsAll(filteredPatternAlphabet)){
					filteredPatternAlphabetAbstractionNameSet.add(maximalElementAbstractionNameMap.get(maximalElement));
				}
			}
			filteredPatternAlphabetAbstractionNameSetMap.put(filteredPatternAlphabet, filteredPatternAlphabetAbstractionNameSet);
		}
		
		return filteredPatternAlphabetAbstractionNameSetMap;
	}

	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		this.logChanged = logChanged;
		if(logChanged){
			patternFilterPanel.setVisible(false);
			abstractionControlPanel.setVisible(false);
			patternInfoTableControlPanel.setVisible(false);
			abstractionResultsPanel.removeAll();
			abstractionSettingsPanel.setVisible(false);
			findAbstractionsButton.setVisible(false);
			patternInfoTableScrollPane.setVisible(false);
			minePatterns = new MinePatterns(this, log);
			mineAbstractions = new MineAbstractions(minePatterns.getEncodedLog(), abstractionResultsPanel);
		}
		
	}

	public void setManager(InteractivityManager manager) {
		this.interactivityManager = manager;
	}

	public InteractivityManager getManager() {
		return interactivityManager;
	}

	public Map<String, SortedSet<EID>> getSelectedElements() {
		boolean isAbstractionsSelected = false;
		Set<Set<String>> selectedMaximalElementSet = new HashSet<Set<String>>();
		Component[] abstractionResultsPanelComponents = abstractionResultsPanel.getComponents();
		for (Component aec : abstractionResultsPanelComponents) {
			if (((AbstractionElementComponent) aec).isSelected()) {
				isAbstractionsSelected = true;
				selectedMaximalElementSet.add(((AbstractionElementComponent) aec)
						.getCurrentEncodedActivitySet());
			}
		}
		
		System.out.println("Selected maximal element Set: "+selectedMaximalElementSet);
		Set<Set<String>> selectedAbstractionPatternAlphabetSet = new HashSet<Set<String>>();
		if(isAbstractionsSelected){
			//Get all the pattern alphabets subsumed in the maximal elements
			Map<Set<String>, Set<Set<String>>> maximalElementSubsumedPatternAlphabetSetMap = mineAbstractions.getMaximalElementSubsumedPatternAlphabetSetMap();
			for(Set<String> maximalElement : selectedMaximalElementSet){
				selectedAbstractionPatternAlphabetSet.add(maximalElement);
				if(maximalElementSubsumedPatternAlphabetSetMap.containsKey(maximalElement))
					selectedAbstractionPatternAlphabetSet.addAll(maximalElementSubsumedPatternAlphabetSetMap.get(maximalElement));
			}
		}
		
		Set<Set<String>> selectedPatternInfoTableSet = new HashSet<Set<String>>();
		int[] selectedRows = table.getSelectedRows();
		for (int selectedRow : selectedRows) {
			selectedPatternInfoTableSet.add(patternInfoTable.getFilteredPatternAlphabetList().get(table.convertRowIndexToModel(selectedRow)));
		}
		
		System.out.println("Selected Pattern Alphabet(s): "+selectedPatternInfoTableSet);
		
		Map<Set<String>, Set<String>> selectedPatternAlphabetPatternSetMap = new HashMap<Set<String>, Set<String>>();
		Set<String> selectedPatternAlphabetPatternSet;
		Map<TreeSet<String>, TreeSet<String>> patternAlphabetPatternSetMap;
		
		if(isBasePatterns){
			patternAlphabetPatternSetMap = minePatterns.getEntireLogCombinedBasePatternAlphabetPatternSetMap();
		}else{
			patternAlphabetPatternSetMap = minePatterns.getEntireLogCombinedPatternAlphabetPatternSetMap();
		}
		System.out.println("Pattern Alphabet Pattern Set Map Size: "+patternAlphabetPatternSetMap.size());
		
		
		if(isAbstractionsSelected){
			for(Set<String> selectedPatternAlphabet : selectedAbstractionPatternAlphabetSet){
				if(patternAlphabetPatternSetMap.containsKey(selectedPatternAlphabet)){
					if(selectedPatternAlphabetPatternSetMap.containsKey(selectedPatternAlphabet)){
						selectedPatternAlphabetPatternSet = selectedPatternAlphabetPatternSetMap.get(selectedPatternAlphabet); 
					}else{
						selectedPatternAlphabetPatternSet = new HashSet<String>();
					}
					selectedPatternAlphabetPatternSet.addAll(patternAlphabetPatternSetMap.get(selectedPatternAlphabet));
					selectedPatternAlphabetPatternSetMap.put(selectedPatternAlphabet, selectedPatternAlphabetPatternSet);
				}else{
					System.out.println("Pattern Alphabet Not in the Map");
				}
			}
		}
		
		for(Set<String> selectedPatternAlphabet : selectedPatternInfoTableSet){
			if(patternAlphabetPatternSetMap.containsKey(selectedPatternAlphabet)){
				if(selectedPatternAlphabetPatternSetMap.containsKey(selectedPatternAlphabet)){
					selectedPatternAlphabetPatternSet = selectedPatternAlphabetPatternSetMap.get(selectedPatternAlphabet); 
				}else{
					selectedPatternAlphabetPatternSet = new HashSet<String>();
				}
				selectedPatternAlphabetPatternSet.addAll(selectedPatternAlphabetPatternSet);
				
				selectedPatternAlphabetPatternSetMap.put(selectedPatternAlphabet, patternAlphabetPatternSetMap.get(selectedPatternAlphabet));
//					System.out.println(patternAlphabetPatternSetMap.get(selectedPatternAlphabet));
			}else{
				System.out.println("Pattern Alphabet Not in the Map");
			}
		}
		
		List<String> encodedTraceList = minePatterns.getEncodedLog().getEncodedTraceList();
		int encodingLength = minePatterns.getEncodedLog().getEncodingLength();
		
		UkkonenSuffixTree suffixTree;
		SortedSet<EID> patternAlphabetEIDSet;
		
		Map<String, SortedSet<EID>> selectedAlphabetEIDSetMap = new HashMap<String, SortedSet<EID>>();
		int traceIndex = 0;
		for(String encodedTrace : encodedTraceList){
			System.out.println(encodedTrace);
			suffixTree = new UkkonenSuffixTree(encodingLength, encodedTrace);
			
			Set<String> decodedPatternAlphabet;
			for(Set<String> patternAlphabet : selectedPatternAlphabetPatternSetMap.keySet()){
				decodedPatternAlphabet = minePatterns.getDecodedAlphabet(patternAlphabet);
				System.out.println("Pattern Alphabet: "+patternAlphabet);
				System.out.println("Decoded Pattern Alphabet: "+decodedPatternAlphabet);
				
				if(selectedAlphabetEIDSetMap.containsKey(decodedPatternAlphabet.toString()))
					patternAlphabetEIDSet = selectedAlphabetEIDSetMap.get(decodedPatternAlphabet.toString());
				else
					patternAlphabetEIDSet = new TreeSet<EID>();
				
				selectedPatternAlphabetPatternSet = selectedPatternAlphabetPatternSetMap.get(patternAlphabet);
				System.out.println("Pattern Set: "+selectedPatternAlphabetPatternSet);
				for(String pattern : selectedPatternAlphabetPatternSet){
					if(suffixTree.noMatches(pattern) > 0){
						
						int[] patternIndices = suffixTree.getMatches(pattern);
						System.out.println("Has Pattern, No Occurrences: "+patternIndices.length);
						for(int patternIndex : patternIndices){
							patternAlphabetEIDSet.add(new EID(traceIndex, patternIndex-1));
							int patternLength = pattern.length()/encodingLength;
							for(int i = 1; i < patternLength; i++){
								patternAlphabetEIDSet.add(new EID(traceIndex, patternIndex-1+i));
							}
						}
					}
				}
				selectedAlphabetEIDSetMap.put(decodedPatternAlphabet.toString(), patternAlphabetEIDSet);
			}
			traceIndex++;
		}
		
		for(String key : selectedAlphabetEIDSetMap.keySet()){
			System.out.println("No. Elements: "+selectedAlphabetEIDSetMap.get(key));
		}
		
		return selectedAlphabetEIDSetMap;
		
	}

	public Map<String, SortedSet<EID>> getVisibleElements() {
		throw new UnsupportedOperationException();
	}

	public JComponent getComponent() {
		return this;
	}

	public double getScale() {
		return 1;
	}

	public void setScale(double newScale) {}

	public void addUpdateListener(UpdateListener listener) {
		updateListenersList.add(listener);
	}

	public void removeUpdateListener(UpdateListener listener) {
		updateListenersList.remove(listener);
	}
}