package org.processmining.plugins.abstractions.visualization;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.processmining.plugins.abstractions.datatypes.ConservednessMetric;
import org.processmining.plugins.abstractions.datatypes.EncodedLog;
import org.processmining.plugins.abstractions.datatypes.MetricPatternAlphabetCount;
import org.processmining.plugins.abstractions.enumtypes.PatternFilter;
import org.processmining.plugins.abstractions.enumtypes.PatternType;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 * 
 */

public class PatternInfoTable {
	Color backgroundColor;
	JTable patternInfoTable;
	EncodedLog encodedLog;
	
	Map<TreeSet<String>, Integer> patternAlphabetCountMap;
	Map<TreeSet<String>, Integer> patternAlphabetInstanceCountMap;
	Map<TreeSet<String>, TreeSet<String>> patternAlphabetPatternSetMap;
	Map<TreeSet<String>, Integer> patternAlphabetConservednessMap;
	Map<PatternFilter, Integer> patternFilterThresholdMap;

	List<TreeSet<String>> filteredPatternAlphabetList;
	Set<TreeSet<String>> deletedPatternAlphabetSet;
	
	public PatternInfoTable(Color backgroundColor){
		this.backgroundColor = backgroundColor;
		patternInfoTable = new JTable();
		patternInfoTable.setBackground(backgroundColor);
	}
	
	public JTable getTable(){
		return patternInfoTable;
	}
	
	public void prepareData(EncodedLog encodedLog, Set<TreeSet<String>> deletedPatternAlphabetSet, MetricPatternAlphabetCount metricPatternAlphabetCount, ConservednessMetric conservernessMetric, Set<PatternType> patternTypeSet, boolean isBasePatterns, boolean ignoreDuplicateTraces, Map<PatternFilter, Integer> patternFilterThresholdMap){
		System.out.println("Preparing Table Data: "+patternTypeSet);
		this.encodedLog = encodedLog;
		this.patternFilterThresholdMap = patternFilterThresholdMap;
		this.deletedPatternAlphabetSet = deletedPatternAlphabetSet;
		if(patternTypeSet.size() > 1){
			if(isBasePatterns){
				patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingCombinedBasePatternAlphabetPatternSetMap();
				if(ignoreDuplicateTraces){
					patternAlphabetCountMap = metricPatternAlphabetCount.getCombinedBasePatternAlphabetNonDuplicateCountMap();
					patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getCombinedBasePatternAlphabetNonDuplicateInstanceCountMap();
					patternAlphabetConservednessMap = conservernessMetric.getCombinedBasePatternAlphabetNonDuplicateConservednessMap();
				}else{
					patternAlphabetCountMap = metricPatternAlphabetCount.getCombinedBasePatternAlphabetCountMap();
					patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getCombinedBasePatternAlphabetInstanceCountMap();
					patternAlphabetConservednessMap = conservernessMetric.getCombinedBasePatternAlphabetConservednessMap();
				}
			}else{
				patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingCombinedPatternAlphabetPatternSetMap();
				if(ignoreDuplicateTraces){
					patternAlphabetCountMap = metricPatternAlphabetCount.getCombinedPatternAlphabetNonDuplicateCountMap();
					patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getCombinedPatternAlphabetNonDuplicateInstanceCountMap();
					patternAlphabetConservednessMap = conservernessMetric.getCombinedPatternAlphabetNonDuplicateConservednessMap();
				}else{
					patternAlphabetCountMap = metricPatternAlphabetCount.getCombinedPatternAlphabetCountMap();
					patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getCombinedPatternAlphabetInstanceCountMap();
					patternAlphabetConservednessMap = conservernessMetric.getCombinedPatternAlphabetConservednessMap();
				}
			}
		}else{
			for(PatternType patternType : patternTypeSet){
				switch(patternType){
				case TandemArrays:
//					System.out.println("Tandem Arrays");
					if(isBasePatterns){
						patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingTandemArrayBasePatternAlphabetPatternSetMap();
						if(ignoreDuplicateTraces){
							patternAlphabetCountMap = metricPatternAlphabetCount.getTandemArrayBasePatternAlphabetNonDuplicateCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getTandemArrayBasePatternAlphabetNonDuplicateInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getTandemArrayBasePatternAlphabetNonDuplicateConservednessMap();
						}else{ 
							patternAlphabetCountMap = metricPatternAlphabetCount.getTandemArrayBasePatternAlphabetCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getTandemArrayBasePatternAlphabetInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getTandemArrayBasePatternAlphabetConservednessMap();
						}
					}else{
						patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingTandemArrayPatternAlphabetPatternSetMap();
						if(ignoreDuplicateTraces){
							patternAlphabetCountMap = metricPatternAlphabetCount.getTandemArrayPatternAlphabetNonDuplicateCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getTandemArrayPatternAlphabetNonDuplicateInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getTandemArrayPatternAlphabetNonDuplicateConservednessMap();
						}else{
							patternAlphabetCountMap = metricPatternAlphabetCount.getTandemArrayPatternAlphabetCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getTandemArrayPatternAlphabetInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getTandemArrayPatternAlphabetConservednessMap();
						}
					}
					break;
				case MaximalRepeats:
//					System.out.println("Maximal Repeats");
					if(isBasePatterns){
						patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingMaximalRepeatBasePatternAlphabetPatternSetMap();
						if(ignoreDuplicateTraces){
							patternAlphabetCountMap = metricPatternAlphabetCount.getMaximalRepeatBasePatternAlphabetNonDuplicateCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getMaximalRepeatBasePatternAlphabetNonDuplicateInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getMaximalRepeatBasePatternAlphabetNonDuplicateConservednessMap();
						}else{
							patternAlphabetCountMap = metricPatternAlphabetCount.getMaximalRepeatBasePatternAlphabetCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getMaximalRepeatBasePatternAlphabetInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getMaximalRepeatBasePatternAlphabetConservednessMap();
						}
					}else{
						patternAlphabetPatternSetMap = metricPatternAlphabetCount.getContributingMaximalRepeatPatternAlphabetPatternSetMap();
						if(ignoreDuplicateTraces){
							patternAlphabetCountMap = metricPatternAlphabetCount.getMaximalRepeatPatternAlphabetNonDuplicateCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getMaximalRepeatPatternAlphabetNonDuplicateInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getMaximalRepeatPatternAlphabetNonDuplicateConservednessMap();
						}else{
							patternAlphabetCountMap = metricPatternAlphabetCount.getMaximalRepeatPatternAlphabetCountMap();
							patternAlphabetInstanceCountMap = metricPatternAlphabetCount.getMaximalRepeatPatternAlphabetInstanceCountMap();
							patternAlphabetConservednessMap = conservernessMetric.getMaximalRepeatPatternAlphabetConservednessMap();
						}
					}
					break;
				}
			}
		}
		prepareTable();
	}

	public Map<TreeSet<String>, TreeSet<String>> getPatternAlphabetPatternSetMap(){
		return patternAlphabetPatternSetMap;
	}
	
	private void prepareTable(){
		int noPatterns = 0;
		for(Set<String> patternAlphabet : patternAlphabetPatternSetMap.keySet())
			noPatterns += patternAlphabetPatternSetMap.get(patternAlphabet).size();
		System.out.println("No. Patterns in Table: "+noPatterns);
		System.out.println("No. Pattern Alphabets: "+patternAlphabetPatternSetMap.size());
		
		int frequencyFilterThreshold = 0, instanceCountPercentageThreshold = 0, conservednessThreshold = 0, alphabetSizeThreshold = 0;
		for(PatternFilter patternFilter : patternFilterThresholdMap.keySet()){
			switch(patternFilter){
			case Frequency:
				frequencyFilterThreshold = patternFilterThresholdMap.get(patternFilter);
				break;
			case InstanceCountPercentage:
				instanceCountPercentageThreshold = patternFilterThresholdMap.get(patternFilter);
				break;
			case Conservedness:
				conservednessThreshold = patternFilterThresholdMap.get(patternFilter);
				break;
			case AlphabetSize:
				alphabetSizeThreshold = patternFilterThresholdMap.get(patternFilter);
				break;
			}
		}
		
		filteredPatternAlphabetList = new ArrayList<TreeSet<String>>();
		for(TreeSet<String> patternAlphabet : patternAlphabetCountMap.keySet()){
			if(deletedPatternAlphabetSet.contains(patternAlphabet))
				continue;
			if(patternAlphabet.size() <= alphabetSizeThreshold)
				continue;
			if(!patternAlphabetPatternSetMap.containsKey(patternAlphabet))
				continue;
			if(patternAlphabetCountMap.get(patternAlphabet) <= frequencyFilterThreshold)
				continue;
			if(!patternAlphabetInstanceCountMap.containsKey(patternAlphabet))
				continue;
			else if(patternAlphabetInstanceCountMap.get(patternAlphabet) <= instanceCountPercentageThreshold)
				continue;
			if(!patternAlphabetConservednessMap.containsKey(patternAlphabet))
				continue;
			else if(patternAlphabetConservednessMap.get(patternAlphabet) <= conservednessThreshold)
				continue;
			filteredPatternAlphabetList.add(patternAlphabet);
		}
		
		noPatterns = 0;
		for(Set<String> patternAlphabet : filteredPatternAlphabetList)
			noPatterns += patternAlphabetPatternSetMap.get(patternAlphabet).size();
		System.out.println("No. Filtered Patterns in Table: "+noPatterns);
		System.out.println("No. Filtered Pattern Alphabets: "+filteredPatternAlphabetList.size());
		
		Object[][] displayData = new Object[filteredPatternAlphabetList.size()][5];
		int index = 0;
		for(TreeSet<String> patternAlphabet : filteredPatternAlphabetList){
			displayData[index][0] = getDecodedAlphabetTableData(patternAlphabet);
			displayData[index][1] = getDecodedPatternTableData(patternAlphabetPatternSetMap.get(patternAlphabet));
			displayData[index][2] = patternAlphabetCountMap.get(patternAlphabet);
			displayData[index][3] = patternAlphabetInstanceCountMap.get(patternAlphabet);
			displayData[index][4] = patternAlphabetConservednessMap.get(patternAlphabet);
			index++;
		}
		
		patternInfoTable.setModel(new PatternTableModel(displayData));
		patternInfoTable.setBackground(backgroundColor);
		
		patternInfoTable.setColumnSelectionAllowed(false);
		patternInfoTable.setRowSelectionAllowed(true);
		patternInfoTable.setFillsViewportHeight(true);
		patternInfoTable.setAutoCreateRowSorter(true);

		patternInfoTable.getColumnModel().getColumn(0).setPreferredWidth(50);
		patternInfoTable.getColumnModel().getColumn(1).setPreferredWidth(200);
		patternInfoTable.getColumnModel().getColumn(2).setPreferredWidth(50);
		patternInfoTable.getColumnModel().getColumn(3).setPreferredWidth(50);
		patternInfoTable.getColumnModel().getColumn(4).setPreferredWidth(50);

		patternInfoTable.getColumnModel().getColumn(0).setCellRenderer(new TextAreaRenderer());
		patternInfoTable.getColumnModel().getColumn(1).setCellRenderer(new TextAreaRenderer());
		patternInfoTable.getColumnModel().getColumn(2).setCellRenderer(new TextAreaRenderer());
		patternInfoTable.getColumnModel().getColumn(3).setCellRenderer(new TextAreaRenderer());
		patternInfoTable.getColumnModel().getColumn(4).setCellRenderer(new TextAreaRenderer());
	}
	
	private String getDecodedAlphabetTableData(TreeSet<String> patternAlphabet){
		StringBuilder strBuilder = new StringBuilder();
		Iterator<String> iter = patternAlphabet.iterator();
		while (iter.hasNext()) {
			strBuilder.append(encodedLog.getCharActivityMap().get(iter.next()));
			if (iter.hasNext()) {
				strBuilder.append("\n");
			}
		}
		return strBuilder.toString();
	}
	
	private String getDecodedPatternTableData(Set<String> patternSet) {
		StringBuilder strBuilder = new StringBuilder();
		int patternLength;
		int encodingLength = encodedLog.getEncodingLength();
		Map<String, String> charActivityMap = encodedLog.getCharActivityMap();
		for (String pattern : patternSet) {
			patternLength = pattern.length() / encodingLength;
			for (int i = 0; i < patternLength; i++) {
				strBuilder.append(charActivityMap.get(pattern.substring(i * encodingLength, (i + 1) * encodingLength)));
				strBuilder.append("\n");
			}
			if (patternSet.size() > 1)
				strBuilder.append("---\n\n");
		}
		return strBuilder.toString();
	}
	
	public List<TreeSet<String>> getFilteredPatternAlphabetList(){
		return filteredPatternAlphabetList;
	}

	public Set<String> getActivitiesInvolvedInPatternSet(){
		Set<String> activitiesInvolvedInPatternSet = new HashSet<String>();
		for(Set<String> patternAlphabet : patternAlphabetPatternSetMap.keySet())
			activitiesInvolvedInPatternSet.addAll(patternAlphabet);
		return activitiesInvolvedInPatternSet;
	}
	
	@SuppressWarnings("serial")
	class PatternTableModel extends AbstractTableModel {
		private String[] columnNames = { "Pattern Alphabet", "Pattern Sequence Set", "Alphabet Count",
				"Instance Count (%)", "Conservedness Metric" };
		
		private Object[][] data;

		public PatternTableModel(Object[][] data) {
			this.data = data;
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return data.length;
		}

		public String getColumnName(int col) {
			return columnNames[col];
		}

		public Object getValueAt(int row, int col) {
			if (data != null && data.length > row)
				return data[row][col];
			else
				return new Object();
		}

		/*
		 * JTable uses this method to determine the default renderer/ editor for
		 * each cell. If we didn't implement this method, then the last column
		 * would contain text ("true"/"false"), rather than a check box.
		 */
		@SuppressWarnings({ "unchecked", "rawtypes" })
		public Class getColumnClass(int c) {
			return getValueAt(0, c).getClass();
		}

		/*
		 * Don't need to implement this method unless your table's editable.
		 */
		public boolean isCellEditable(int row, int col) {
			//Note that the data/cell address is constant,
			//no matter where the cell appears onscreen.
			if (col > 1) {
				return false;
			} else {
				return true;
			}
		}

		/*
		 * Don't need to implement this method unless your table's data can
		 * change.
		 */
		public void setValueAt(Object value, int row, int col) {

			data[row][col] = value;
		}
	}

	@SuppressWarnings( { "unchecked", "rawtypes", "serial" })
	private class TextAreaRenderer extends JTextArea implements TableCellRenderer {
		private final DefaultTableCellRenderer adaptee = new DefaultTableCellRenderer();
		/** map from table to map of rows to map of column heights */

		private final Map cellSizes = new HashMap();

		public TextAreaRenderer() {
			setLineWrap(true);
			setWrapStyleWord(true);
		}

		public Component getTableCellRendererComponent(JTable table, Object obj, boolean isSelected, boolean hasFocus,
				int row, int column) {
			// set the colours, etc. using the standard for that platform
			adaptee.getTableCellRendererComponent(table, obj, isSelected, hasFocus, row, column);
			setForeground(adaptee.getForeground());
			setBackground(adaptee.getBackground());
			setBorder(adaptee.getBorder());
			setFont(adaptee.getFont());
			setText(adaptee.getText());

			// This line was very important to get it working with JDK1.4
			TableColumnModel columnModel = table.getColumnModel();
			setSize(columnModel.getColumn(column).getWidth(), 100000);
			int height_wanted = (int) getPreferredSize().getHeight();
			addSize(table, row, column, height_wanted);
			height_wanted = findTotalMaximumRowSize(table, row);
			if (height_wanted != table.getRowHeight(row)) {
				table.setRowHeight(row, height_wanted);
			}
			return this;
		}

		private void addSize(JTable table, int row, int column, int height) {
			Map rows = (Map) cellSizes.get(table);
			if (rows == null) {
				cellSizes.put(table, rows = new HashMap());
			}
			Map rowheights = (Map) rows.get(new Integer(row));
			if (rowheights == null) {
				rows.put(new Integer(row), rowheights = new HashMap());
			}
			rowheights.put(new Integer(column), new Integer(height));
		}

		/**
		 * Look through all columns and get the renderer. If it is also a
		 * TextAreaRenderer, we look at the maximum height in its hash table for
		 * this row.
		 */
		private int findTotalMaximumRowSize(JTable table, int row) {
			int maximum_height = 0;
			Enumeration columns = table.getColumnModel().getColumns();
			while (columns.hasMoreElements()) {
				TableColumn tc = (TableColumn) columns.nextElement();
				TableCellRenderer cellRenderer = tc.getCellRenderer();
				if (cellRenderer instanceof TextAreaRenderer) {
					TextAreaRenderer tar = (TextAreaRenderer) cellRenderer;
					maximum_height = Math.max(maximum_height, tar.findMaximumRowSize(table, row));
				}
			}
			return maximum_height;
		}

		private int findMaximumRowSize(JTable table, int row) {
			Map rows = (Map) cellSizes.get(table);
			if (rows == null)
				return 0;
			Map rowheights = (Map) rows.get(new Integer(row));
			if (rowheights == null)
				return 0;
			int maximum_height = 0;
			for (Iterator it = rowheights.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				int cellHeight = ((Integer) entry.getValue()).intValue();
				maximum_height = Math.max(maximum_height, cellHeight);
			}
			return maximum_height;
		}
	}

	
}
