package org.processmining.plugins.abstractions.visualization;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.WindowConstants;

import org.processmining.plugins.abstractions.enumtypes.RemoveAbstractionStrategy;
import org.processmining.plugins.abstractions.swingx.ScrollableGridLayout;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author R. P. Jagadeesh Chandra 'JC' Bose
 * @date 15 May 2011 Eindhoven University of Technology (TU/e)
 * 
 *       Reproducing or using whole or part of this code/application is to be done with proper
 *       acknowledgments by citing "R. P. Jagadeesh Chandra Bose and Wil M. P.
 *       van der Aalst. Abstractions in Process Mining: A Taxonomy of Patterns.
 *       Proceedings of the International Conference on Business Process
 *       Management (BPM). Volume 5701 of LNCS., Springer-Verlag (2009) pages
 *       159-175"
 * 
 */

@SuppressWarnings("serial")
public class RemoveAbstractionsEditDialog extends JDialog{
	PatternAbstractionFrame owner;
	
	JRadioButton removeOnlyMaximalElementRadioButton;
	JRadioButton removeUniquelySubsumedAlphabetRadioButton;
	
	RemoveAbstractionStrategy returnValue = RemoveAbstractionStrategy.NONE;
	
	public RemoveAbstractionsEditDialog(PatternAbstractionFrame owner){
		super(JOptionPane.getFrameForComponent(owner), "Remove Abstractions");
		this.owner = owner;
		
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		Dimension d = owner.getPreferredSize();
		setLocation(new Point(d.width/2, d.height/2));
		
		buildUI();
		
		pack();
//		d = getSize();

//		setSize(300, d.height);
	}
	
	private void buildUI(){
		JPanel removeOptionsPanel = SlickerFactory.instance().createRoundedPanel();
		ScrollableGridLayout removeOptionsPanelLayout = new ScrollableGridLayout(removeOptionsPanel, 1, 3, 0, 0);
		
		removeOptionsPanelLayout.setRowFixed(0, true);
		removeOptionsPanelLayout.setRowFixed(1, true);
		removeOptionsPanelLayout.setRowFixed(2, true);
		
		removeOptionsPanel.setLayout(removeOptionsPanelLayout);
		
		removeOnlyMaximalElementRadioButton = SlickerFactory.instance().createRadioButton("Remove Only the Selected Maximal Element         ");
		removeUniquelySubsumedAlphabetRadioButton = SlickerFactory.instance().createRadioButton("Remove Uniquely Subsumed Alphabets (of the Maximal Element) Also      ");
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(removeOnlyMaximalElementRadioButton);
		buttonGroup.add(removeUniquelySubsumedAlphabetRadioButton);
		removeOnlyMaximalElementRadioButton.setSelected(true);
		
		removeOptionsPanelLayout.setPosition(removeOnlyMaximalElementRadioButton, 0, 0);
		removeOptionsPanel.add(removeOnlyMaximalElementRadioButton);
		removeOptionsPanelLayout.setPosition(removeUniquelySubsumedAlphabetRadioButton, 0, 1);
		removeOptionsPanel.add(removeUniquelySubsumedAlphabetRadioButton);
		
		JPanel buttonPanel = SlickerFactory.instance().createRoundedPanel();
		JButton okButton = SlickerFactory.instance().createButton("OK");
		JButton cancelButton = SlickerFactory.instance().createButton("Cancel");
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(removeOnlyMaximalElementRadioButton.isSelected())
					returnValue = RemoveAbstractionStrategy.MAXIMAL_ELEMENT_ONLY;
				else
					returnValue = RemoveAbstractionStrategy.UNIQUELY_SUBSUMED_ALPHABETS;
				owner.removeAbstractions(returnValue);
				dispose();
			}
		});
		
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				returnValue = RemoveAbstractionStrategy.NONE;
				dispose();
			}
		});
		
		removeOptionsPanelLayout.setPosition(buttonPanel, 0, 2);
		removeOptionsPanel.add(buttonPanel);
		getContentPane().add(removeOptionsPanel, BorderLayout.CENTER);
	}

	public RemoveAbstractionStrategy getReturnValue(){
		return returnValue;
	}
}
